/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */
#pragma once

namespace NuoDB
{

/**
 * @brief Representation of a Binary Large Object value to be stored as a column
 * value in a row of a table.
 *
 * The representation in C++ of an SQL BLOB value. An SQL BLOB is a built-in data 
 * type that stores a Binary Large Object as a column value in a row of a database 
 * table. 
 *
 * See <a href="http://doc.nuodb.com/Latest/Default.htm#SQL-Binary-Types.htm"><strong>SQL Binary Types</strong></a>
 * in the NuoDB documentation for more information about this data type.
 */
class Blob
{
protected:
    virtual ~Blob() {}

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the 
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

    /** 
     * @brief   Return number of bytes in the BLOB value specified by the BLOB 
     *          object
     * 
     * @return  the length in bytes of the BLOB
     */
    virtual int length() = 0;

    /** 
     * @brief   Retrieves an array of bytes representing all or part of the BLOB 
     *          value specified by the BLOB object.
     *
     * @param   offset Ordinal position of the first byte in the BLOB to be 
     *          extracted, where the BLOB starts at position 1
     * @param   length Number of consecutive bytes to get. Value must be 0 or 
     *          greater.
     * @param   buffer BLOB buffer.
     */
    virtual void getBytes(int offset, int length, unsigned char* buffer) = 0;

    /** 
     * @brief   Writes specified bytes to BLOB value for the BLOB object. 
     *
     * @param   length Number of bytes to be written
     * @param   buffer BLOB buffer.
     */
    virtual void setBytes(int length, const unsigned char* buffer) = 0;

    /** 
     * @brief   Writes specified bytes to the end of the BLOB value specified 
     *          by the BLOB object. 
     *
     * @param   length Number of bytes to be written
     * @param   buffer BLOB buffer.
     */
    virtual void appendBytes(int length, const unsigned char* buffer) = 0;
};

} // end of namespace NuoDB
