/* Copyright 2013-2015 NuoDB, Inc. All Rights Reserved. */

#pragma once

#include <stdint.h>
#include <stdarg.h>

#include "NuoRemote/SqlExports.h"

/**
 * @namespace NuoDB
 * @brief Namespace for all of the %NuoDB C++ API
 */
namespace NuoDB
{
class Blob;
class CallableStatement;
class Clob;
class DatabaseMetaData;
class PreparedStatement;
class Properties;
class SQLWarning;
class Savepoint;
class Statement;

typedef void (LogListener)(int, const char*, void*);

static const int    CONNECTION_VERSION  = 1;
static const int    TRANSACTION_NONE                = 0;
static const int    TRANSACTION_READ_UNCOMMITTED    = 1;
static const int    TRANSACTION_READ_COMMITTED      = 2;
static const int    TRANSACTION_REPEATABLE_READ     = 4;
static const int    TRANSACTION_SERIALIZABLE        = 8;


/**
 * @brief Establish a connection to the %NuoDB database.
 *
 * In order to perform persistent work, you need to establish a connection to the
 * %NuoDB database.  This is done by creating a Connection object.
 *
 * See
 * <a href="http://doc.nuodb.com/Latest/Default.htm#Using-the-NuoDB-CPP-Driver.htm"><strong>Using the NuoDB C++ Driver</strong></a>.
 */

class Connection
{
protected:
    virtual ~Connection() {}

public:
    /**
     * @brief   Closes the connection to the database
     *
     * If the socket to the database is still open, this will send a
     * CloseConnection message to the Transaction Engine (TE) and then close the
     * socket. In either case it will delete this Connection object.
     */
    virtual void    close() = 0;

    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the
     *          reference count is 0, delete the object
     *
     * It is recommended you call close() on the Connection object rather than
     * release().
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

    /**
     * @brief   Create a Statement object used to send SQL operations to the
     *          database
     *
     * The Statement object is not parameterized, unlike a PreparedStatement object.
     *
     * @return  a pointer to a new Statmeent object
     *
     * @throw   SQLException
     */
    virtual Statement* createStatement() = 0;

    /**
     * @brief   Create a PreparedStatement object for sending parameterized
     *          statements to the database
     *
     * A SQL statement with or without IN parameters can be pre-compiled and
     * stored in a PreparedStatement object. This object can then be used to
     * efficiently execute this statement multiple times.  This method will send
     * the statement to the database for precompilation and will only be compiled
     * once.
     *
     * @param   sql a valid, optionally parameterized, SQL statmenet
     *
     * @return  a pointer to a new PreparedStatement object
     *
     * @throw   SQLException
     */
    virtual PreparedStatement*  prepareStatement(const char* sql) = 0;

    /**
     * @brief   Create a CallableStatement object for calling database stored
     *          procedures
     *
     * The CallableStatement object provides methods for setting up IN and
     * OUT parameters for stored procedures, and methods for executing the call
     * to the stored procedure.
     *
     * @param   sql a valid, optionally parameterized, SQL statmenet
     *
     * @return  a pointer to a new CallableStatement object
     *
     * @throw   SQLException
     */
    virtual CallableStatement*  prepareCall(const char* sql) = 0;

    /* not implemented */
    virtual const char*         nativeSQL(const char* sql) = 0;

    /**
     * @brief   Specify if statements related to this connections should
     *          be auto-committed.
     *
     * If a connection is in auto-commit mode, then all its SQL statements will
     * be executed and committed as individual transactions. Otherwise, its SQL
     * statements are grouped into transactions that are terminated by a call to
     * either the method commit or the method rollback. By default, new
     * connections are in auto-commit mode.
     *
     * @param   autoCommit  If true, set autoCommit on and if false, turn
     *                      autoCommit off
     *
     * @throw   SQLException
     */
    virtual void setAutoCommit(bool autoCommit) = 0;

    /**
     * @brief   Determine if the connection has the auto-commit flag set.
     *
     * @return  True if autoCommit is on and false otherwise
     *
     * @throw   SQLException
     */
    virtual bool getAutoCommit() = 0;

    /**
     * @brief   Commit any outstanding SQL statements.
     *
     * Makes all changes made since the previous commit/rollback permanent and
     * releases any database locks currently held by this Connection object. This
     * method should be used only when auto-commit mode has been disabled.
     *
     * @throw  SQLException
     */
    virtual void commit() = 0;

    /**
     * @brief   Roll back any outstanding SQL statements.
     *
     * Undoes all changes made in the current transaction and releases any
     * database locks currently held by this Connection object. This method
     * should be used only when auto-commit mode has been disabled.
     *
     * @throw   SQLException
     */
    virtual void    rollback() = 0;

    /* not implemented */
    virtual bool    isClosed() = 0;

    /**
     * @brief   Retrieve metadata for the database associated to this
     *          connection
     *
     * The metadata includes information about the database's tables, its
     * supported SQL grammar, its stored procedures, the capabilities of this
     * connection, and so on.
     *
     * The Connection object manages the memory allocated in the DatabaseMetaData
     * object and the caller does not have to do so.
     *
     * @return  pointer to a DatabaseMetaData object
     *
     * @throw   SQLException
     */
    virtual DatabaseMetaData*   getMetaData() = 0;

    /**
     * @brief   Puts this connection in read-only mode as a hint to enable
     *          database optimizations.
     *
     * @throw SQLException
     */
    virtual void                setReadOnly(bool readOnly) = 0;

    /**
     * @brief   Retrieves whether this Connection object is in read-only mode.
     *
     * @throw   SQLException
     */
    virtual bool                isReadOnly() = 0;

    /* not implemented */
    virtual void                setCatalog(const char* catalog) = 0;

    /* not implemented */
    virtual const char*         getCatalog() = 0;

    /**
     * @brief   Change the transaction isolation level for this Connection object
     *          to the one specified.
     *
     * Valid transaction isolation levels are:
     * <ul>
     * <li>NuoDB::transactionWriteCommitted    = 5</li>
     * <li>NuoDB::transactionReadCommitted     = 2</li>
     * <li>NuoDB::transactionConsistentRead    = 7</li>
     * <li>NuoDB::transactionSerializable      = 8</li>
     * </ul>
     *
     * @throw   SQLException
     */
    virtual void                setTransactionIsolation(int level) = 0;

    /**
     * @brief   Retrieves this Connection object's current transaction isolation
     *          level
     *
     * @return  the Connection's trancation level - one of:
     * <ul>
     * <li>NuoDB::transactionWriteCommitted    = 5</li>
     * <li>NuoDB::transactionReadCommitted     = 2</li>
     * <li>NuoDB::transactionConsistentRead    = 7</li>
     * <li>NuoDB::transactionSerializable      = 8</li>
     * </ul>
     *
     * @throw   SQLException
     */
    virtual int                 getTransactionIsolation() = 0;

    /* not implemented */
    virtual SQLWarning*         getWarnings() = 0;

    /* not implemented */
    virtual void                clearWarnings() = 0;

    /* not implemented */
    /* c-api-suffix=with_type */
    virtual Statement*          createStatement(int resultSetType, int resultSetConcurrency) = 0;

    /* not implemented */
    /* c-api-suffix=with_type */
    virtual PreparedStatement*  prepareStatement(const char* sql, int resultSetType, int resultSetConcurrency) = 0;

    /* not implemented */
    /* c-api-suffix=with_type */
    virtual CallableStatement*  prepareCall(const char* sql, int resultSetType, int resultSetConcurrency) = 0;

    /* not implemented */
    virtual void                setHoldability(int holdability) = 0;

    /* not implemented */
    virtual int                 getHoldability() = 0;

    /* not implemented */
    virtual Savepoint*          setSavepoint() = 0;

    /* not implemented */
    /* c-api-suffix=with_name */
    virtual Savepoint*          setSavepoint(const char* name) = 0;

    /* not implemented */
    /* c-api-suffix=to */
    virtual void                rollback(Savepoint* savepoint) = 0;

    /* not implemented */
    virtual void                releaseSavepoint(Savepoint* savepoint) = 0;

    /* not implemented */
    /* c-api-suffix=with_holdability */
    virtual Statement*          createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) = 0;

    /* not implemented */
    /* c-api-suffix=with_holdability */
    virtual PreparedStatement*  prepareStatement(const char* sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) = 0;

    /* not implemented */
    /* c-api-suffix=with_holdability */
    virtual CallableStatement*  prepareCall(const char* sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) = 0;

    /**
     * @brief   Creates a default PreparedStatement object that has the capability
     *          to retrieve auto-generated keys.
     *
     * The autoGeneratedKeys parameter tells the %NuoDB server whether it should
     * make auto-generated keys available for retrieval. This parameter is ignored
     * if the SQL statement is not an INSERT statement, or an SQL statement able
     * to return auto-generated keys.
     *
     * @param   sql                 a valid, optionally parameterized, SQL statmenet
     * @param   autoGeneratedKeys   a flag indicating whether auto-generated keys
     *                              should be made available for retrieval: one of
     *                              Statement::RETURN_GENERATED_KEYS or
     *                              Statement::NO_GENERATED_KEYS
     *
     * @return  a pointer to a PreparedStatement object
     *
     * @throw SQLException
     */
    /* c-api-suffix=with_keys */
    virtual PreparedStatement*  prepareStatement(
                                    const char* sql,
                                    int         autoGeneratedKeys) = 0;

    /**
     * @brief   Creates a default PreparedStatement object capable of returning
     *          the auto-generated keys designated by the given array.
     *
     * This array contains the indexes of the columns in the target table that
     * contain the auto-generated keys that should be made available. The array
     * will be ignored if the SQL statement is not an INSERT statement, or some
     * other SQL statement able to return auto-generated keys.  An SQL statement
     * with or without IN parameters can be pre-compiled and stored in a
     * PreparedStatement object. This object can then be used to efficiently
     * execute this statement multiple times.
     *
     * @param   sql                 a valid, optionally parameterized, SQL statmenet
     * @param   columnIndexes       an array of column indexes indicating the
     *                              columns that should be returned from the
     *                              inserted row or rows
     * @param   numberIndexes       the number of indexes contained in the
     *                              columnIndexes array
     *
     * @return  a pointer to a PreparedStatement object
     *
     * @throw SQLException
     */
    /* c-api-suffix=with_indexes */
    virtual PreparedStatement*  prepareStatement(
                                    const char* sql,
                                    int*        columnIndexes,
                                    int         numberIndexes) = 0;

    /**
     * @brief   Creates a default PreparedStatement object capable of returning
     *          the auto-generated keys designated by the given array.
     *
     * This array contains the names of the columns in the target table that
     * contain the auto-generated keys that should be returned. The array will
     * be ignored if the SQL statement is not an INSERT statement, or some other
     * SQL statement able to return auto-generated keys.  An SQL statement with
     * or without IN parameters can be pre-compiled and stored in a
     * PreparedStatement object. This object can then be used to efficiently
     * execute this statement multiple times.
     * @param   sql                 a valid, optionally parameterized, SQL statmenet
     * @param   columnNames         an array of column names indicating the
     *                              columns that should be returned from the
     *                              inserted row or rows
     * @param   numberColumnNames   the number of names contained in the
     *                              columnNames array
     *
     * @return  a pointer to a PreparedStatement object
     *
     * @throw SQLException
     */
    /* c-api-suffix=with_names */
    virtual PreparedStatement*  prepareStatement(
                                    const char*     sql,
                                    const char**    columnNames,
                                    int             numberColumnNames) = 0;

    /**
     * @brief   Create an SQL CLOB object.
     *
     * It is the responsibility of the caller to call release() on the CLOB when
     * the caller is finished with it.
     *
     * @return  a pointer to a Clob object
     *
     * @throw   SQLException
     */
    virtual Clob* createClob() = 0;

    /**
     * @brief   Create an SQL BLOB object.
     *
     * It is the responsibility of the caller to call release() on the BLOB when
     * the caller is finished with it.
     *
     * @return  a pointer to a Blob object
     *
     * @throw   SQLException
     */
    virtual Blob* createBlob() = 0;

    /* not implemented */
    virtual bool        isValid(int timeout) = 0;

    /* not implemented */
    /* c-api-suffix=by_name */
    virtual void        setClientInfo(const char* name, const char* value) = 0;

    /* not implemented */
    virtual void        setClientInfo(Properties* properties) = 0;

    /* not implemented */
    /* c-api-suffix=by_name */
    virtual const char* getClientInfo(const char* name) = 0;

    /* not implemented */
    virtual Properties* getClientInfo() = 0;

    /* not implemented */
    virtual void        setSchema(const char* schema) = 0;

    /**
     * @brief   Retrieves this Connection object's current schema name.
     *
     * @return  the Connection's current schema
     *
     * @throw SQLException
     */
    virtual const char* getSchema() = 0;

    /* not implemented */
    virtual int getNetworkTimeout() = 0;

    /**
     * @brief   Determine if the client has a valid connection to a %NuoDB TE
     *
     * @return  true if the socket which communicates to the server exists and
     *          is open
     */
    virtual bool    isConnected() = 0;

    /* not implemented */
    virtual void    prepareTransaction() = 0;

    /**
     * @brief   Send a ping message to the server.
     *
     * If no exception occurs, the server is alive.
     *
     * @throw SQLException
     */
    virtual void    ping() = 0;

    /* not implemented */
    virtual int     hasRole(const char* schemaName, const char* roleName) = 0;

    /**
     * @brief       Given an already existing Connection object, open a database
     *              with the given set of properties
     *
     * This is an alternative to calling create(). It should be used in conjunction
     * with Connection::create() with no arguemnts.  This allows you to set up a
     * Properties object with key/value database properties and use that to open
     * the database.
     *
     * @param   database    a point to a const char string representing the database name
     * @param   context     a pointer to a Properties class which contains
     *                      key/value pairs specifying properties for opening the database
     *
     * @throws  SQLException
     */
    virtual void    openDatabase(const char* database, Properties* context) = 0;

    /**
     * @brief       Allocate a Properties object for the purpose of passing to
     *              openDatabase()
     *
     * The user is responsible for freeing the memory allocated by
     * allocProperties() when they are done with the Properties object.
     *
     * @return  a pointer to an empty Properties object
     */
    virtual Properties* allocProperties() = 0;

    /**
     * @brief   Returns the connection version
     *
     * @return  the connection version
     */
    virtual int         objectVersion() = 0;

    /**
     * @brief   Returns a point to a copy of this Connection object
     *
     * This creates a new Connection object with the same properties as this
     * Connection object and opens the database creating a second independent
     * connection.
     *
     * @return  a pointer to a copy of this Connection object
     *
     * @throw   SQLException
     */
    virtual Connection* clone() = 0;

    /* not implemented */
    virtual void        validate(int flags) = 0;

    /* not implemented */
    virtual void    addLogListener(int mask, LogListener fn, void* arg) = 0;

    /* not implemented */
    virtual void    deleteLogListener(LogListener fn, void* arg) = 0;

    /* not implemented */
    virtual const char* analyze(int mask) = 0;

    /* unused */
    virtual int32_t     setTraceFlags(int32_t mask) = 0;

    /* not implemented */
    virtual void        setAttribute(const char* name, const char* value) = 0;

    /**
     * @brief   Specify a non-default encoding for this client connection.
     *
     * %NuoDB uses ICU (International Components for Unicode) for converting
     * strings from the local client environment to UTF-8. %NuoDB Transaction
     * Engines require all strings to be UTF-8. setConverter() allows you to
     * specify a non-default character set table (encoding) that is being used
     * by the client. This may have options appended to the string. IANA alias
     * character set names, IBM CCSIDs starting with "ibm-", Windows codepage
     * numbers starting with "windows-" are frequently ' used for this parameter.  See
     * <a href="http://icu-project.org/apiref/icu4c/ucnv_8h.html#a485c78d4149165f504effa2287717e41">
     * <strong>ICU ucnv_open()</strong></a> for more information.
     *
     * @param   encoding    Name of a non-default character set table.  If this
     *                      parameter is NULL, the default converter will be used.
     */
    virtual void        setConverter(const char* encoding) = 0;

    /**
     * @brief   Specify a non-default encoding for this client connection and
     *          set it as the default for the connection.
     *
     * %NuoDB uses ICU (International Components for Unicode) for converting
     * strings from the local client environment to UTF-8. %NuoDB Transaction
     * Engines require all strings to be UTF-8. setDefaultConverter() allows you
     * to specify a non-default character set table (encoding) that is being used
     * by the client. This may have options appended to the string. IANA alias
     * character set names, IBM CCSIDs starting with "ibm-", Windows codepage
     * numbers starting with "windows-" are frequently used for this parameter.  See
     * <a href="http://icu-project.org/apiref/icu4c/ucnv_8h.html#a485c78d4149165f504effa2287717e41">
     * <strong>ICU ucnv_open()</strong></a> for more information.
     *
     * setDefaultConverter() will set the connections default converter to that
     * for the given encoding. Future calls to getDefaultEncoding() will return
     * this new default encoding.
     *
     * @param   encoding    Name of a non-default character set table.  If this
     *                      parameter is NULL, the default converter will be used.
     */
    virtual void        setDefaultConverter(const char* encoding) = 0;

    /**
     * @brief   Return the name of the default encoding for this Connection.
     *
     * @return  the name of the default encoding
     */
    virtual const char* getDefaultEncoding() = 0;

    /**
     * @brief   Return the name of the active encoding for this Connection which
     *          may not be the same as the default encoding for this Connection
     *
     * @return  the name of the active encoding
     */
    virtual const char* getActiveEncoding() = 0;

    /**
     * @brief   Convert the given, presumably UTF-8 string, to the local encoding
     *          using the Connection's active encoding.  See
     *          Connection::getActiveEncoding().
     *
     * @return  a pointer to the converted string
     *
     * @throw   SQLException
     */
    virtual char*       convertUtf8ToLocal(const char* s) = 0;

    /* not implemented */
    virtual int         getEffectivePlatformVersionFromServer() = 0;

    virtual int         getProtocolVersion() = 0;
    /**
     * @brief       Attempts to establish a connection to the given %NuoDB database.
     *
     * Static method to create a connection as in the following example:
     *
     * <pre>
     * Connection::create(
     *          "dbname",
     *          "dbaUser",
     *          "dbaPassword",
     *          1,
     *          "schema", "hello" );
     * </pre>
     *
     * @param   url                 the database name
     * @param   user                the database user to use for this connection
     * @param   password            the database user's password
     * @param   numberProperties    the number of key/value pairs, representing
     *                              connection properties, that will follow
     *
     * @return  a pointer to a Connection object
     *
     * @throw   SQLException
     */
    /* c-api-suffix=v */
    NUODB_SQLAPI static Connection* create(
                                        const char* url,
                                        const char* user,
                                        const char* password,
                                        int         numberProperties,
                                        ...);

    /**
     * @brief       Attempts to establish a connection to the given %NuoDB database.
     *
     * Static method to create a connection as in the following example:
     *
     * <pre>
     * Connection::create(
     *          "dbname",
     *          "dbaUser",
     *          "dbaPassword",
     *          argcount,
     *          argvalist );
     * </pre>
     *
     * @param   url                 the database name
     * @param   user                the database user to use for this connection
     * @param   password            the database user's password
     * @param   numberProperties    the number of arguments in the args va_list
     * @param   args                a va_list populated with key/value connection
     *                              properties, perhaps passed in on the command
     *                              line
     *
     * @return  a pointer to a Connection object
     *
     * @throw   SQLException
     */
    /* c-api-suffix=va_list */
    NUODB_SQLAPI static Connection* create(
                                        const char* url,
                                        const char* user,
                                        const char* password,
                                        int         numberProperties,
                                        va_list     args);

    /**
     * @brief       Create a Connection object to be used later with
     *              openDatabase()
     *
     * Create a generic Connection object, on which you would later call
     * openDatabase() to open a specific database.
     *
     * @return  a pointer to a Connection object
     *
     * @throw SQLException
     */
    NUODB_SQLAPI static Connection* create();

    /**
     * @brief       Create a Connection object to be used later with
     *              openDatabase(), and which assumes a UTF-8 encoding
     *
     * Create a generic Connection object, on which you would later call
     * openDatabase() to open a specific database. This overloading will assume
     * strings being passed from the client to the %NuoDB TE will be UTF-8
     * despite the encoding set in the user's local environment.
     *
     * @return  a pointer to a Connection object
     *
     * @throw SQLException
     */
    NUODB_SQLAPI static Connection* createUtf8();
};

/* not used... */
static const int    traceBooleans      = 1;
static const int    traceTriggers      = 2;

static const int    opTraceAll     = 1;
static const int    opShutdown     = 2;
static const int    opShutdownImmed    = 4;
static const int    opCancelRequest    = 8;
static const int    opClone            = 16;
static const int    opCreateShadow = 32;
static const int    opInstallLicense   = 64;

static const int    validatePages      = 1;
static const int    validateRepair = 2;
static const int    validateBlobs      = 4;
static const int    validateSpecial    = 8;
static const int    validateMinutia    = 16;

static const int    statementMaxSortLimit      = 1;
static const int    connectionMaxSortLimit = 2;
static const int    statementMaxRecordsLimit   = 3;
static const int    connectionMaxRecordsLimit  = 4;
/* ...end not used */

/**
 * Transactions are not supported.
 */
static const int    transactionNone            = 0;

/* internal only */
static const int    transactionReadUncommitted = 1; // Dirty reads, non-repeatable reads and phantom reads can occur.

/**
 * Each transaction always reads the most recently committed version of a record.
 * See <a href="http://doc.nuodb.com/Latest/Default.htm#Supported-Transaction-Isolation-Levels.htm"><strong>
 * Supported Transaction Isolation Levels</strong></a> for more information.
 */
static const int    transactionReadCommitted   = 2;

/**
 * This will also cause the same behavior as CONSISTENT_READ which is the default
 * %NuoDB transaction isolation level. Each transaction sees a snapshot of the
 * database as of the start of the transaction, plus changes made by that transaction.
 * See <a href="http://doc.nuodb.com/Latest/Default.htm#Supported-Transaction-Isolation-Levels.htm"><strong>
 * Supported Transaction Isolation Levels</strong></a> for more information.
 */
static const int    transactionRepeatableRead  = 4;

/**
 * This is kept for backward compatibility and will also cause the same behavior
 * as CONSISTENT_READ which is the default %NuoDB transaction isolation level.
 * Each transaction sees a snapshot of the database as of the start of the
 * transaction, plus changes made by that transaction only. See
 * <a href="http://doc.nuodb.com/Latest/Default.htm#Supported-Transaction-Isolation-Levels.htm"><strong>
 * Supported Transaction Isolation Levels</strong></a> for more information.
 */
static const int    transactionSerializable    = 8;

/**
 * For read operations, each transaction sees the state of the database as it was
 * when the transaction started, plus its own changes,  similar to CONSISTENT_READ.
 * For write operations, including SELECT...FOR UPDATE, UPDATE and DELETE), each
 * transaction reads the most recently committed version of a record, similar to
 * READ_COMMITTED. See
 * <a href="http://doc.nuodb.com/Latest/Default.htm#Supported-Transaction-Isolation-Levels.htm"><strong>
 * Supported Transaction Isolation Levels</strong></a> for more information.
 */
static const int    transactionWriteCommitted  = 5;

/**
 * This is the default %NuoDB transaction isolation level. Each transaction sees a
 * snapshot of the database as of the start of the transaction, plus changes made
 * by that transaction only. See
 * <a href="http://doc.nuodb.com/Latest/Default.htm#Supported-Transaction-Isolation-Levels.htm"><strong>
 * Supported Transaction Isolation Levels</strong></a> for more information.
 */
static const int    transactionConsistentRead  = 7;

/**
 * This instructs Statement::analyze() to produce index statistics about the
 * records contained in the index
 */
static const int    analyzeStatementRecord     = 1;

/**
 * This instructs Statement::analyze() to include information about the
 * execution plan for the statement in the index statistics
 */
static const int    analyzeStatementExecution      = 2;
} // end of namespace NuoDB

// alternative to Connection::create() methods
extern "C" NUODB_SQLAPI NuoDB::Connection* NuoDB_createConnection();
extern "C" NUODB_SQLAPI NuoDB::Connection* NuoDB_createUtf8Connection();
extern "C" NUODB_SQLAPI NuoDB::Connection* NuoDB_getDatabaseConnection(const char* url, const char* user, const char* password, int numberProperties, ...);


// Deprecated versions.
// GCC 4.4 and below don't support messages to the deprecated attribute.
extern "C" NUODB_SQLAPI NuoDB::Connection* createConnection()
    __attribute__((deprecated));
extern "C" NUODB_SQLAPI NuoDB::Connection* createUtf8Connection()
    __attribute__((deprecated));
extern "C" NUODB_SQLAPI NuoDB::Connection* getDatabaseConnection(const char* url, const char* user, const char* password, int numberProperties, ...)
    __attribute__((deprecated));
