/* Copyright 2013-2015 NuoDB, Inc. All Rights Reserved. */

#pragma once

#include <stdint.h>

#include "NuoRemote/DatabaseMetaDataConstants.h"

namespace NuoDB
{
class ResultSet;
class Properties;
class RowIdLifetime;
class Connection;

/**
 * @brief Retrieve information about the database
 *
 * Some DatabaseMetaData methods return lists of information in the form of
 * ResultSet objects. Regular ResultSet methods, such as getString and getInt,
 * can be used to retrieve the data from these ResultSet objects. If a given
 * form of metadata is not available, an empty ResultSet will be returned.
 *
 * Some DatabaseMetaData methods take arguments that are String patterns. These
 * arguments all have names such as fooPattern. Within a pattern String, "%"
 * means match any substring of 0 or more characters, and "_" means match any
 * one character. Only entries matching the search pattern are returned.
 * If a search pattern argument is set to null, that argument's criterion will
 * be dropped from the search.
 */

class DatabaseMetaData
{
protected:
    virtual ~DatabaseMetaData() {}

public:
    /**
     * @brief   Retrieves whether the current user can call all the procedures
     *          returned by the method getProcedures().
     *
     * Currently unimplemented and always returns false
     *
     * @return  true if all procedures are callable and false otherwise
     */
    virtual bool            allProceduresAreCallable() = 0;

    /**
     * @brief   Retrieves whether the current user can use all the tables
     *          returned by the method getTables() in a SELECT statement.
     *
     * Currently unimplemented and always returns true
     *
     * @return  true if the current user has SELECT privilege for all tables
     *          returned by getTables()
     */
    virtual bool            allTablesAreSelectable() = 0;

    /**
     * @brief   Returns the database name followed by option host and port
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getURL() except that there is no real connection URL
     * for a %NuoDB C++ API client database connection. This just returns the
     * database name followed by an optional host and port specification, e.g.
     * "dbname[@host[:port]]".
     *
     * @return  the client connection URL
     */
    virtual const char*     getURL() = 0;

    /**
     * @brief   Returns current user for the client connection
     *
     * @return  the name of the current user
     */
    virtual const char*     getUserName() = 0;

    /**
     * @brief   Retrieves whether this database is in read-only mode
     *
     * @return  true, if the connection is in read only mode
     *          (Connection::setReadOnly) and false otherwise
     *
     * @throw   SQLException
     */
    virtual bool            isReadOnly() = 0;

    /**
     * @brief   Retrieves whether NULL values are sorted high.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%nullsAreSortedHigh(). Sorted high means that NULL values
     * sort higher than any other value. %NuoDB always sorts NULL values at the
     * start, regardless of sort order and therefore this method always returns
     * false.  See nullsAreSortedAtStart().
     *
     * @return  false
     */
    virtual bool            nullsAreSortedHigh() = 0;

    /**
     * @brief   Retrieves whether NULL values are sorted low.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%nullsAreSortedLow(). Sorted low means that NULL values
     * sort lower than any other value. %NuoDB always sorts NULL values at the
     * start, regardless of sort order and therefore this method always returns
     * false.  See nullsAreSortedAtStart().
     *
     * @return  false
     */
    virtual bool            nullsAreSortedLow() = 0;

    /**
     * @brief   Retrieves whether NULL values are sorted at the start regardless
     *          of sort order.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%nullsAreSortedAtStart(). %NuoDB always sorts NULL values
     * at the start, regardless of sort order, and therefore this method always
     * returns true.
     *
     * @return  true
     */
    virtual bool            nullsAreSortedAtStart() = 0;

    /**
     * @brief   Retrieves whether NULL values are sorted at the end regardless
     *          of sort order.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%nullsAreSortedAtEnd(). %NuoDB always sorts NULL values
     * at the start, regardless of sort order, and therefore this method always
     * returns false.
     *
     * @return  false
     */
    virtual bool            nullsAreSortedAtEnd() = 0;

    /**
     * @brief   Retrieves the database product name
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getDatabaseProductName().
     *
     * @return  the string "NuoDB"
     */
    virtual const char*     getDatabaseProductName() = 0;

    /**
     * @brief   Retrieves the database product version
     *
     * This method returns the three part release version number for the
     * database, followed by a build number, e.g. m1.m2.m3-b, where m1 is the
     * major release number, m2 is the minor release number, m3 is the
     * maintenance release number and b is the build number.  This is relative
     * to the Transaction Engine to which the client is connected. That is, in
     * the case of rolling upgrade, where there are database processes from
     * different %NuoDB releases, this method may return different results.
     *
     * @return  the string m1.m2.m3-b where m1 is the major release number, m2
     *          is the minor release number, m3 is the maintenance release number
     *          and b is the build number.
     */
    virtual const char*     getDatabaseProductVersion() = 0;

    /**
     * @brief   Retrieves the name of the %NuoDB C++ Driver
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getDriverName() and simply returns the string "NuoDB"
     *
     * @return  "NuoDB"
     */
    virtual const char*     getDriverName() = 0;

    /**
     * @brief   Retrieves the release version string of this %NuoDB client
     *
     * This method returns the three part release version number for the
     * client, followed by a build number, e.g. m1.m2.m3-b, where m1 is the
     * major release number, m2 is the minor release number, m3 is the
     * maintenance release number and b is the build number.  This is release
     * version number for the client and not for the actual database which could
     * be different.
     *
     * @return  the string m1.m2.m3-b where m1 is the major release number, m2
     *          is the minor release number, m3 is the maintenance release number
     *          and b is the build number.
     */
    virtual const char*     getDriverVersion() = 0;

    /**
     * @brief   Retrieves the major version number of this %NuoDB client
     *
     * This method returns the major release version number for the client. This
     * is not for the version number for the actual database which could be
     * different.
     *
     * @return  the major version number for the client
     */
    virtual int             getDriverMajorVersion() = 0;

    /**
     * @brief   Retrieves the minor version number of this %NuoDB client
     *
     * This method returns the minor release version number for the client. This
     * is not for the version number for the actual database which could be
     * different.
     *
     * @return  the minor version number for the client
     */
    virtual int             getDriverMinorVersion() = 0;

    /**
     * @brief   Returns whether the database uses local files
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%usesLocalFiles(). It always returns false.
     *
     * @return  false
     */
    virtual bool            usesLocalFiles() = 0;

    /**
     * @brief   Retrieves whether this database uses a file for each table.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%usesLocalFilesPerTable(). It always returns false.
     *
     * @return  false
     */
    virtual bool            usesLocalFilePerTable() = 0;

    /**
     * @brief   Retrieves whether this database treats mixed case unquoted SQL
     *          identifiers as case sensitive and as a result stores them in
     *          mixed case.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsMixedCaseIdentifiers(). It always returns false.
     * %NuoDB does not support mixed case identifiers.  See
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  false
     */
    virtual bool            supportsMixedCaseIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats mixed case unquoted SQL
     *          identifiers as case insensitive and stores them in uppercase.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%storesUpperCaseIdentifiers(). It always returns true.
     * %NuoDB always stores identifiers as uppercase.  See
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  true
     */
    virtual bool            storesUpperCaseIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats mixed case unquoted SQL
     *          identifiers as case insensitive and stores them in lowercase.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%storesLowerCaseIdentifiers(). It always returns false.
     * %NuoDB always stores identifiers as uppercase.  See
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  false
     */
    virtual bool            storesLowerCaseIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats mixed case unquoted SQL
     *          identifiers as case insensitive and stores them in mixed case.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%storesMixedCaseIdentifiers(). It always returns false.
     * %NuoDB always stores identifiers as uppercase.  See
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  false
     */
    virtual bool            storesMixedCaseIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats mixed case quoted SQL
     *          identifiers as case sensitive and as a result stores them in
     *          mixed case.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsMixedCaseIdentifiers(). It always returns true.
     * %NuoDB always stores <strong>quoted</strong> identifiers as mixed case.
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  true
     */
    virtual bool            supportsMixedCaseQuotedIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats uppercase quoted SQL
     *          identifiers as case sensitive and as a result stores them in
     *          mixed case.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%storesUpperCaseIdentifiers(). It always returns true.
     * %NuoDB always stores uppercase <strong>quoted</strong> identifiers as uppercase.
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  true
     */
    virtual bool            storesUpperCaseQuotedIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats lowercase quoted SQL
     *          identifiers as case sensitive and as a result stores them in
     *          mixed case.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%storesLowerCaseIdentifiers(). It always returns true.
     * %NuoDB always stores lowercase <strong>quoted</strong> identifiers as lowercase.
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  true
     */
    virtual bool            storesLowerCaseQuotedIdentifiers() = 0;

    /**
     * @brief   Retrieves whether this database treats mixed case quoted SQL
     *          identifiers as case sensitive and as a result stores them in
     *          mixed case.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%storesMixedCaseIdentifiers(). It always returns true.
     * %NuoDB always stores <strong>quoted</strong> identifiers as mixed case.
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  true
     */
    virtual bool            storesMixedCaseQuotedIdentifiers() = 0;

    /**
     * @brief   Retrieves the string used to quote SQL identifiers. This method
     *          returns a space " " if identifier quoting is not supported.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getIdentiferQuoteString(). It always returns the string
     * with one double-quote.
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  "\""
     */
    virtual const char*     getIdentifierQuoteString() = 0;

    /* not implemented */
    virtual const char*     getSQLKeywords() = 0;

    /* not implemented */
    virtual const char*     getNumericFunctions() = 0;

    /* not implemented */
    virtual const char*     getStringFunctions() = 0;

    /* not implemented */
    virtual const char*     getSystemFunctions() = 0;

    /* not implemented */
    virtual const char*     getTimeDateFunctions() = 0;

    /**
     * @brief   Retrieves the string that can be used to escape special ASCII
     *          characters.
     *
     * This always returns the string "`".
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Rules-for-Specifying-NuoDB-Identifier-Names.htm"><strong>
     * Rules for Specifying NuoDB Identifier Names</strong></a> for more information.
     *
     * @return  "`"
     */
    virtual const char*     getSearchStringEscape() = 0;

    /* not implemented */
    virtual const char*     getExtraNameCharacters() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports ALTER TABLE with add column.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsAlterTableWithAddColumn(). It always returns true.
     *
     * @return  true
     */
    virtual bool            supportsAlterTableWithAddColumn() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports ALTER TABLE with drop column.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsAlterTableWithDropColumn(). It always returns true.
     *
     * @return  true
     */
    virtual bool            supportsAlterTableWithDropColumn() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports column aliasing.
     *
     * This means the SQL AS clause can be used to provide names for computed
     * columns or to provide alias names for columns as required.  This method
     * is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsAlterColumnAliasing(). It always returns true.
     *
     * @return  true
     */
    virtual bool            supportsColumnAliasing() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports concatenations between NULL
     *          and non-NULL values being NULL.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsNullPlusNullIsNull(). It always returns true.
     *
     * @return  true
     */
    virtual bool            nullPlusNonNullIsNull() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports the JDBC scalar function
     *          CONVERT for the conversion of one JDBC type to another.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsConvert(). It always returns false.
     *
     * @return  false
     */
    virtual bool            supportsConvert() = 0;

    /* c-api-suffix=from_to */
    virtual bool            supportsConvert(int fromType, int toType) = 0;

    /**
     * @brief   Retrieves whether this database supports table correlation names.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsTableCorrelationNames(). It always returns false.
     *
     * @return  false
     */
    virtual bool            supportsTableCorrelationNames() = 0;

    /**
     * @brief   Retrieves whether, when table correlation names are supported,
     *          they are restricted to being different from the names of the tables.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsDifferentTableCorrelationNames(). It always
     * returns false.
     *
     * @return  false
     */
    virtual bool            supportsDifferentTableCorrelationNames() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports expressions in ORDER BY lists.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsExpressionsInOrderBy(). It always
     * returns false.
     *
     * @return  false
     */
    virtual bool            supportsExpressionsInOrderBy() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports using a column that is not in
     *          the SELECT statement in an ORDER BY clause.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsOrderByUnrelated(). It always
     * returns false.
     *
     * @return  false
     */
    virtual bool            supportsOrderByUnrelated() = 0;

    /* not implemented */
    virtual bool            supportsGroupBy() = 0;

    /* not implemented */
    virtual bool            supportsGroupByUnrelated() = 0;

    /* not implemented */
    virtual bool            supportsGroupByBeyondSelect() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports specifying a LIKE escape clause.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsLikeEscapeClause(). It always
     * returns false.
     *
     * @return  false
     */
    virtual bool            supportsLikeEscapeClause() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports getting multiple ResultSet
     *          objects from a single call to the method execute.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsMultipleResultSets(). It always
     * returns true.
     *
     * @return  true
     */
    virtual bool            supportsMultipleResultSets() = 0;

    /* not implemented */
    virtual bool            supportsMultipleTransactions() = 0;

    /**
     * @brief   Retrieves whether columns in %NuoDB may be defined as non-nullable.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsNonNullableColumns(). It always
     * returns true.
     *
     * @return  true
     */
    virtual bool            supportsNonNullableColumns() = 0;

    /* not implemented */
    virtual bool            supportsMinimumSQLGrammar() = 0;

    /* not implemented */
    virtual bool            supportsCoreSQLGrammar() = 0;

    /* not implemented */
    virtual bool            supportsExtendedSQLGrammar() = 0;

    /* not implemented */
    virtual bool            supportsANSI92EntryLevelSQL() = 0;

    /* not implemented */
    virtual bool            supportsANSI92IntermediateSQL() = 0;

    /* not implemented */
    virtual bool            supportsANSI92FullSQL() = 0;

    /* not implemented */
    virtual bool            supportsIntegrityEnhancementFacility() = 0;

    /* not implemented */
    virtual bool            supportsOuterJoins() = 0;

    /* not implemented */
    virtual bool            supportsFullOuterJoins() = 0;

    /* not implemented */
    virtual bool            supportsLimitedOuterJoins() = 0;

    /**
     * @brief   Retrieves %NuoDB's preferred term for "schema".
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getSchemaTerm(). It always
     * returns "schema"
     *
     * @return  "schema"
     */
    virtual const char*     getSchemaTerm() = 0;

    /**
     * @brief   Retrieves %NuoDB's preferred term for "procedure"
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getProcedureTerm(). It always
     * returns "procedure"
     *
     * @return  "procedure"
     */
    virtual const char*     getProcedureTerm() = 0;

    /**
     * @brief   Retrieves %NuoDB's preferred term for "catalog"
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%getCatalogTerm(). It always
     * returns "database"
     *
     * @return  "database"
     */
    virtual const char*     getCatalogTerm() = 0;

    /* not implemented */
    virtual bool            isCatalogAtStart() = 0;

    /* not implemented */
    virtual const char*     getCatalogSeparator() = 0;

    /* not implemented */
    virtual bool            supportsSchemasInDataManipulation() = 0;

    /* not implemented */
    virtual bool            supportsSchemasInProcedureCalls() = 0;

    /* not implemented */
    virtual bool            supportsSchemasInTableDefinitions() = 0;

    /* not implemented */
    virtual bool            supportsSchemasInIndexDefinitions() = 0;

    /* not implemented */
    virtual bool            supportsSchemasInPrivilegeDefinitions() = 0;

    /* not implemented */
    virtual bool            supportsCatalogsInDataManipulation() = 0;

    /* not implemented */
    virtual bool            supportsCatalogsInProcedureCalls() = 0;

    /* not implemented */
    virtual bool            supportsCatalogsInTableDefinitions() = 0;

    /* not implemented */
    virtual bool            supportsCatalogsInIndexDefinitions() = 0;

    /* not implemented */
    virtual bool            supportsCatalogsInPrivilegeDefinitions() = 0;

    /* not implemented */
    virtual bool            supportsPositionedDelete() = 0;

    /* not implemented */
    virtual bool            supportsPositionedUpdate() = 0;

    /* not implemented */
    virtual bool            supportsSelectForUpdate() = 0;

    /* not implemented */
    virtual bool            supportsStoredProcedures() = 0;

    /* not implemented */
    virtual bool            supportsSubqueriesInComparisons() = 0;

    /* not implemented */
    virtual bool            supportsSubqueriesInExists() = 0;

    /* not implemented */
    virtual bool            supportsSubqueriesInIns() = 0;

    /* not implemented */
    virtual bool            supportsSubqueriesInQuantifieds() = 0;

    /* not implemented */
    virtual bool            supportsCorrelatedSubqueries() = 0;

    /* not implemented */
    virtual bool            supportsUnion() = 0;

    /* not implemented */
    virtual bool            supportsUnionAll() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports keeping cursors open across
     *          commits
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsOpenCursorsAcrossCommit(). It always
     * returns true.
     *
     * @return  true
     */
    virtual bool            supportsOpenCursorsAcrossCommit() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports keeping cursors open across
     *          rollbacks
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsOpenCursorsAcrossRollback(). It always
     * returns true.
     *
     * @return  true
     */
    virtual bool            supportsOpenCursorsAcrossRollback() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports keeping statements open across
     *          commits.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportOpenStatementsAcrossCommit(). It always
     * returns true.
     *
     * @return  true
     */
    virtual bool            supportsOpenStatementsAcrossCommit() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports keeping statements open across
     *          rollbacks.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsOpenStatementsAcrossRollback(). It always
     * returns true.
     *
     * @return  true
     */
    virtual bool            supportsOpenStatementsAcrossRollback() = 0;

    /* not implemented */
    virtual int             getMaxBinaryLiteralLength() = 0;

    /* not implemented */
    virtual int             getMaxCharLiteralLength() = 0;

    /* not implemented */
    virtual int             getMaxColumnNameLength() = 0;

    /* not implemented */
    virtual int             getMaxColumnsInGroupBy() = 0;

    /* not implemented */
    virtual int             getMaxColumnsInIndex() = 0;

    /* not implemented */
    virtual int             getMaxColumnsInOrderBy() = 0;

    /* not implemented */
    virtual int             getMaxColumnsInSelect() = 0;

    /* not implemented */
    virtual int             getMaxColumnsInTable() = 0;

    /* not implemented */
    virtual int             getMaxConnections() = 0;

    /* not implemented */
    virtual int             getMaxCursorNameLength() = 0;

    /* not implemented */
    virtual int             getMaxIndexLength() = 0;

    /* not implemented */
    virtual int             getMaxSchemaNameLength() = 0;

    /* not implemented */
    virtual int             getMaxProcedureNameLength() = 0;

    /* not implemented */
    virtual int             getMaxCatalogNameLength() = 0;

    /* not implemented */
    virtual int             getMaxRowSize() = 0;

    /* not implemented */
    virtual bool            doesMaxRowSizeIncludeBlobs() = 0;

    /* not implemented */
    virtual int             getMaxStatementLength() = 0;

    /* not implemented */
    virtual int             getMaxStatements() = 0;

    /* not implemented */
    virtual int             getMaxTableNameLength() = 0;

    /* not implemented */
    virtual int             getMaxTablesInSelect() = 0;

    /* not implemented */
    virtual int             getMaxUserNameLength() = 0;

    /**
     * @brief   Retrieves %NuoDB's default transaction isolation level.
     *
     * The %NuoDB default transactaction isolation level is CONSISTENT_READ (7)
     * or SERIALIZAZBLE (8). This method always returns SERIALIZABLE.
     *
     * @return  SERIALIZABLE (8)
     */
    virtual int             getDefaultTransactionIsolation() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports transactions.
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsTransactions(). It always returns true.
     *
     * @return  true
     */
    virtual bool            supportsTransactions() = 0;

    /**
     * @brief   Retrieves whether %NuoDB supports the given transaction
     *          isolation level.
     *
     * @param   level   one of the following transaction isolation levels:<br />
     *  NuoDB::transactionReadCommitted<br />
     *  NuoDB::transactionRepeatableRead<br />
     *  NuoDB::transactionWriteCommitted<br />
     *  NuoDB::transactionConsistentRead<br />
     *  NuoDB::transactionSerializable<br />
     *
     * This method is implemented for compatibility with JDBC
     * %DatabaseMetaData#%supportsTransactionIsolationLevel().
     * This method returns true for the following levels:<br />
     *  NuoDB::transactionReadCommitted<br />
     *  NuoDB::transactionWriteCommitted<br />
     *  NuoDB::transactionConsistentRead<br />
     *  NuoDB::transactionSerializable<br />
     * and false otherwise. See
     * <a href="http://doc.nuodb.com/Latest/Default.htm#Supported-Transaction-Isolation-Levels.htm"><strong>
     * Supported Transaction Isolation Levels</strong></a> for more information.
     *
     * @return  true if the isolation level is supported and false otherwise.
     *
     * @throw   SQLException
     */
    virtual bool            supportsTransactionIsolationLevel(int level) = 0;

    /* not implemented */
    virtual bool            supportsDataDefinitionAndDataManipulationTransactions() = 0;

    /* not implemented */
    virtual bool            supportsDataManipulationTransactionsOnly() = 0;

    /* not implemented */
    virtual bool            dataDefinitionCausesTransactionCommit() = 0;

    /* not implemented */
    virtual bool            dataDefinitionIgnoredInTransactions() = 0;

    /**
     * @brief   Retrieves a description of the stored procedures available in
     *          %NuoDB
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getProcedures().  Only procedure descriptions matching
     * the schema and procedure name criteria are returned. They are ordered by
     * PROCEDURE_SCHEM and PROCEDURE_NAME.
     *
     * Each procedure description has the the following columns:
     *
     * <ol>
     * <li>PROCEDURE_CAT    const char* => procedure catalog (may be null)<br />
     *      Always set to empty string by %NuoDB</li>
     * <li>PROCEDURE_SCHEM  const char* => procedure schema (may be null)</li>
     * <li>PROCEDURE_NAME   const char* => procedure name</li>
     * <li>reserved for future use</li>
     * <li>reserved for future use</li>
     * <li>reserved for future use</li>
     * <li>REMARKS          const char* => always empty string in %NuoDB</li>
     * <li>PROCEDURE_TYPE   short => kind of procedure:<br/>
     *      %NuoDB always returns 1 here, NuoDB::procedureNoResult, which means the
     *      procedure does not return a value.</li>
     * <li>SPECIFIC_NAME    const char* => The name which uniquely identifies
     *                      this procedure within its schema. For %NuoDB, this is
     *                      always the same as PROCEDURE_NAME</li>
     * </ol>
     *
     *  A user may not have permissions to execute any of the procedures that
     *  are returned by getProcedures()
     *
     * @param   catalog                 provided for JDBC compatibility and
     *                                  ignored for %NuoDB
     * @param   schemaPattern           a schema name pattern - must match the
     *                                  schema name as it is stored in the
     *                                  database; "" retrieves those without a
     *                                  schema; null means that the schema name
     *                                  should not be used to narrow the search
     * @param   procedureNamePattern    a procedure name pattern - must match the
     *                                  procedure name as it is stored in the
     *                                  database
     *
     * @return  ResultSet with one row per matching procedure
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getProcedures(
                                const char* catalog,
                                const char* schemaPattern,
                                const char* procedureNamePattern ) = 0;

    /**
     * @brief   Retrieves a description of the database's stored procedure
     *          parameter and result columns
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getProcedureColumns().  Only descriptions matching the
     * schema, procedure and parameter name criteria are returned. The return
     * value, if any, is first. Next are the parameter descriptions in call order.
     * The column descriptions follow in column number order.
     *
     * Each row in the ResultSet is a parameter description or column description
     * with the following columns:
     *
     * <ol>
     * <li>PROCEDURE_CAT    const char* => procedure catalog (may be null)<br />
     *      Always set to empty string by %NuoDB</li>
     * <li>PROCEDURE_SCHEM  const char* => procedure schema (may be null)</li>
     * <li>PROCEDURE_NAME   const char* => procedure name</li>
     * <li>COLUMN_NAME      const char* => column/parameter name</li>
     * <li>COLUMN_TYPE      short => kind of column/parameter:
     *      <ul>
     *      <li>NuoDB::procedureColumnUnknown (0) - nobody knows</li>
     *      <li>NuoDB::procedureColumnIn (1) - IN parameter</li>
     *      <li>NuoDB::procedureColumnInOut (2) - INOUT parameter</li>
     *      <li>NuoDB::procedureColumnOut (3) - OUT parameter</li>
     *      <li>NuoDB::procedureColumnReturn (4) - procedure return value</li>
     *      <li>NuoDB::procedureColumnResult (5) - result column in ResultSet</li>
     *      </ul></li>
     * <li>DATA_TYPE        int => the type code defined by NuoDB::SqlType</li>
     * <li>TYPE_NAME        const char* => SQL type name</li>
     * <li>PRECISION        int => precision</li>
     * <li>LENGTH           int => length in bytes of data</li>
     * <li>SCALE            short => scale - null is returned for data types </li>
     *                      where SCALE is not applicable.</li>
     * <li>RADIX            short => %NuoDB always returns 0</li>
     * <li>NULLABLE         short => can it contain NULL, %NuoDB always returns 1
     *                      (NuoDB::procedureNullable)</li>
     * <li>REMARKS          const char* => always empty string in %NuoDB</li>
     * <li>COLUMN_DEF       const char* => default value for the column, which
     *                      should be interpreted as a string when the value is
     *                      enclosed in single quotes (may be null)<br/>
     *      %NuoDB always returns NULL meaning a default value was not specified</li>
     * <li>SQL_DATA_TYPE    int => reserved for future use</li>
     * <li>SQL_DATETIME_SUB int => reserved for future use</li>
     * <li>CHAR_OCTET_LENGTH int => the maximum length of binary and character
     *                      based columns. For any other datatype the returned
     *                      value is a NULL</li>
     * <li>ORDINAL_POSITION int => the ordinal position, starting from 1, for
     *                      the input and output parameters for a procedure. A
     *                      value of 0 is returned if this row describes the
     *                      procedure's return value. For result set columns, it
     *                      is the ordinal position of the column in the result
     *                      set starting from 1. If there are multiple result
     *                      sets, the column ordinal positions are implementation
     *                      defined.</li>
     * <li>IS_NULLABLE      const char* => ISO rules are used to determine the
     *                      nullability for a column.<br/>
     *      %NuoDB always returns an empty string, meaning the nullability for the
     *      column is unknown</li>
     * <li>SPECIFIC_NAME    const char* => the name which uniquely identifies
     *                      this procedure within its schema.<br />
     *      %NuoDB always returns the PROCEDURE_NAME here</li>
     * </ol>
     *
     *  A user may not have permissions to execute any of the procedures that
     *  are returned by getProcedures()
     *
     * @param   catalog                 provided for JDBC compatibility and
     *                                  ignored for %NuoDB
     * @param   schemaPattern           a schema name pattern - must match the
     *                                  schema name as it is stored in the
     *                                  database; "" retrieves those without a
     *                                  schema; null means that the schema name
     *                                  should not be used to narrow the search
     * @param   procedureNamePattern    a procedure name pattern - must match the
     *                                  procedure name as it is stored in the
     *                                  database
     * @param   columnNamePattern       a column name pattern - must match the
     *                                  column name as it is stored in the database
     *
     * @return  ResultSet with each row representing a stored procedure parameter or column
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getProcedureColumns(
                                const char* catalog,
                                const char* schemaPattern,
                                const char* procedureNamePattern,
                                const char* columnNamePattern) = 0;

    /**
     * @brief   Retrieves a description of the tables available in the given
     *          database.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getTables().  Only table descriptions matching the
     * schema, table name and type criteria are returned. They are ordered by
     * TABLE_TYPE, TABLE_SCHEM and TABLE_NAME.
     *
     * Each table description has the following columns:
     *
     * <ol>
     * <li>TABLE_CAT                const char* => table catalog (may be null)<br />
     *          Always set to empty string by %NuoDB</li>
     * <li>TABLE_SCHEM              const char* => table schema (may be null)</li>
     * <li>TABLE_NAME               const char* => table name</li>
     * <li>TABLE_TYPE               const char* => table type<br />
     *      Types include: "TABLE", "VIEW", "SYSTEM TABLE", "SYSTEM PSEUDO TABLE"</li>
     * <li>REMARKS                  const char* => always empty string in %NuoDB</li>
     * <li>TYPE_CAT                 const char* => the types catalog (may be null)<br />
     *      Always NULL for %NuoDB</li>
     * <li>TYPE_SCHEM               const char* => the types schema (may be null)<br />
     *      Always NULL for %NuoDB</li>
     * <li>TYPE_NAME                const char* => type name (may be null)<br />
     *      Always NULL for %NuoDB</li>
     * <li>SELF_REFERENCING_COL_NAME const char* => name of the designated
     *                              "identifier" column of a typed table (may be null)<br />
     *      Always NULL for %NuoDB</li>
     * <li>REF_GENERATION           const char* => specifies how values in
     *                              SELF_REFERENCING_COL_NAME are created.<br />
     *      Always NULL for %NuoDB</li>
     * </ol>
     *
     * @param   catalog             provided for JDBC compatibility and ignored
     *                              for %NuoDB
     * @param   schemaPattern       a schema name pattern - must match the schema
     *                              name as it is stored in the database; ""
     *                              retrieves those without a schema; null means
     *                              that the schema name should not be used to narrow
     *                              the search
     * @param   tableNamePattern    a table name pattern - must match the table
     *                              name as it is stored in the database
     * @param   types               a list of table types to include; these types
     *                              must be from the list of table types returned from
     *                              getTableTypes(); null returns all types
     *
     * @return  ResultSet   with each row representing a matching table.
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getTables(
                                const char*     catalog,
                                const char*     schemaPattern,
                                const char*     tableNamePattern,
                                int             typeCount,
                                const char**    types) = 0;

    /**
     * @brief   Retrieves the schema names available in this database.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getSchemas().
     *
     * The schema columns are:
     *
     * <ol>
     * <li>TABLE_SCHEM      const char* => schema name</li>
     * <li>TABLE_CATALOG    const char* => catalog name (may be null)<br />
     *          Always set to empty string by %NuoDB</li>
     * </ol>
     *
     * @return  ResultSet   with each row representing a schema.
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getSchemas() = 0;

    /**
     * @brief   Retrieves the catalog names available in this database.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getCatalogs(). %NuoDB does not support the concept of
     * catalogs.
     *
     * The columns are:
     *
     * <ol>
     * <li>TABLE_CATALOG    const char* => catalog name (may be null)<br />
     *          Always set to empty string by %NuoDB</li>
     * </ol>
     *
     * @return  ResultSet   %NuoDB always returns an empty ResultSet
     */
    virtual ResultSet*      getCatalogs() = 0;

    /**
     * @brief   Retrieves the table types available in this database.
     *
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getTableTypes(). The results are ordered by table type.
     *
     * The columns include:
     *
     * <ol>
     * <li>TABLE_TYPE   const char* => table type. %NuoDB supports the following
     *                  table types:<br />
     *      <ul>
     *      <li>SYSTEM PSEUDO TABLE</li>
     *      <li>SYSTEM TABLE</li>
     *      <li>TABLE</li>
     *      <li>VIEW</li>
     *      </ul></li>
     * </ol>
     *
     * @return  ResultSet   with each row representing a table type.
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getTableTypes() = 0;

    /**
     * @brief   Retrieves a description of table columns available in the database
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getColumns(). Only column descriptions matching the
     * schema, table and column name criteria are returned. They are ordered by
     * ORDINAL_POSITION.
     *
     * Each column description has the following columns:
     *
     * <ol>
     * <li>TABLE_CAT        const char* => table catalog (may be null)<br />
     *          Always set to empty string by %NuoDB</li>
     * <li>TABLE_SCHEM      const char* => table schema (may be null)</li>
     * <li>TABLE_NAME       const char* => table name</li>
     * <li>COLUMN_NAME      const char* => column name</li>
     * <li>DATA_TYPE        int => the type code defined by NuoDB::SqlType</li>
     * <li>TYPE_NAME        const char* => Data source dependent type name</li>
     * <li>COLUMN_SIZE      int => column size</li>
     * <li>BUFFER_LENGTH    is not used</li>
     * <li>DECIMAL_DIGITS   int => the number of fractional digits</li>
     * <li>NUM_PREC_RADIX   int => Radix; %NuoDB always returns 10</li>
     * <li>NULLABLE         int => is NULL allowed<br />
     *      %NuoDB returns NuoDB::columnNoNulls (0) if NULL might not be allowed
     *      and (NuoDB::columnNullable) 1 if NULL is definitely allowed</li>
     * <li>REMARKS          const char* => always empty string in %NuoDB</li>
     * <li>COLUMN_DEF       const char* => default value for the column</li>
     * <li>SQL_DATA_TYPE    int => unused</li>
     * <li>SQL_DATETIME_SUB int => unused</li>
     * <li>CHAR_OCTET_LENGTH int => for char types the maximum number of bytes
     *                      in the column</li>
     * <li>ORDINAL_POSITION int => index of column in table (starting at 1)</li>
     * <li>IS_NULLABLE      const char* => ISO rules are used to determine the
     *                      nullability for a column.<br />
     *      <ul>
     *      <li>YES --- if the column can include NULLs</li>
     *      <li>NO --- if the column cannot include NULLs</li>
     *      </ul></li>
     * <li>SCOPE_CATALOG    const char* => %NuoDB always returns null</li>
     * <li>SCOPE_SCHEMA     const char* => %NuoDB always returns null</li>
     * <li>SCOPE_TABLE      const char* => %NuoDB always returns null</li>
     * <li>SOURCE_DATA_TYPE short => %NuoDB always returns null</li>
     * <li>IS_AUTOINCREMENT const char* => Indicates whether this column is auto
     *                      incremented<br />
     *      <ul>
     *      <li>YES --- if the column is auto incremented</li>
     *      <li>NO --- if the column is not auto incremented</li>
     *      </ul></li>
     * <li>IS_GENERATEDCOLUMN const char* => Indicates whether this is a generated
     *                       column
     *      <ul>
     *      <li>YES --- if this a generated column
     *      <li>NO --- if this not a generated column
     *      </ul></li>
     * </ol>
     *
     * @param   catalog             provided for JDBC compatibility and ignored
     *                              for %NuoDB
     * @param   schemaPattern       a schema name pattern - must match the schema
     *                              name as it is stored in the database; ""
     *                              retrieves those without a schema; null means
     *                              that the schema name should not be used to narrow
     *                              the search
     * @param   tableNamePattern    a table name pattern - must match the table
     *                              name as it is stored in the database
     * @param   columnNamePattern   a column name pattern - must match the column
     *                              name as it is stored in the database
     *
     * @return  ResultSet   with each row representing a matching table.
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getColumns(
                                const char* catalog,
                                const char* schemaPattern,
                                const char* tableNamePattern,
                                const char* columnNamePattern) = 0;

    /* not implemented */
    virtual ResultSet*      getColumnPrivileges(const char* catalog, const char* schema, const char* table, const char* columnNamePattern) = 0;

    /* not implemented */
    virtual ResultSet*      getTablePrivileges(const char* catalog, const char* schemaPattern, const char* tableNamePattern) = 0;

    /* not implemented */
    virtual ResultSet*      getBestRowIdentifier(const char* catalog, const char* schema, const char* table, int scope, bool nullable) = 0;

    /* not implemented */
    virtual ResultSet*      getVersionColumns(const char* catalog, const char* schema, const char* table) = 0;

    /**
     * @brief   Retrieves a description of the given table's primary key
     *          columns. They are ordered by KEY_SEQ.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getPrimaryKeys().
     *
     * Each primary key column description has the following columns:
     *
     * <ol>
     * <li>TABLE_CAT    const char* => table catalog (may be null)<br />
     *          Always set to empty string by %NuoDB</li>
     * <li>TABLE_SCHEM  const char* => table schema (may be null)</li>
     * <li>TABLE_NAME   const char* => table name</li>
     * <li>COLUMN_NAME  const char* => column name</li>
     * <li>KEY_SEQ      short => sequence number within primary key (a value of
     *                  1 represents the first column of the primary key, a value
     *                  of 2 would represent the second column within the primary
     *                  key).</li>
     * <li>PK_NAME      const char* => primary key name (may be null)</li>
     * </ol>
     *
     * @param   catalog     provided for JDBC compatibility and ignored for %NuoDB
     * @param   schema      a schema name - must match the schema name as it is
     *                      stored in the database; "" retrieves those without a
     *                      schema; null means that the schema name should not be
     *                      used to narrow the search
     * @param   table       a table name - must match the table name as it is
     *                      stored in the database
     *
     * @return  ResultSet   with each row representing a primary key column
     *                      description.
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getPrimaryKeys(
                                const char* catalog,
                                const char* schema,
                                const char* table) = 0;

    /**
     * @brief   Retrieves a description of the primary key columns that are
     *          referenced by the given table's foreign key columns (the primary
     *          keys imported by a table). They are ordered by PKTABLE_SCHEM,
     *          PKTABLE_NAME, and KEY_SEQ.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getImportedKeys().
     *
     * Each imported key column description has the following columns:
     *
     * <ol>
     * <li>PKTABLE_CAT      const char* => primary key table catalog (may be
     *                      null)<br /> Always set to empty string by %NuoDB</li>
     * <li>PKTABLE_SCHEM    const char* => primary key table schema (may be
     *                      null)</li>
     * <li>PKTABLE_NAME     const char* => primary key table name</li>
     * <li>PKCOLUMN_NAME    const char* => primary key column name</li>
     * <li>FKTABLE_CAT      const char* => foreign key table catalog (may be
     *                      null)<br /> Always set to empty string by %NuoDB</li>
     * <li>FKTABLE_SCHEM    const char* => foreign key table schema (may be
     *                      null)</li>
     * <li>FKTABLE_NAME     const char* => foreign key table name</li>
     * <li>FKCOLUMN_NAME    const char* => foreign key column name</li>
     * <li>KEY_SEQ          short => sequence number within foreign key (a value
     *                      of 1 represents the first column of the primary key,
     *                      a value of 2 would represent the second column within
     *                      the primary key).</li>
     * <li>UPDATE_RULE      short => What happens to a foreign key when the
     *                      primary key is updated. Values are:
     *      <ul>
     *      <li>NuoDB::importedKeyCascade (0)- change imported key to agree with
     *      primary key update</li>
     *      <li>NuoDB::importedKeySetNull (2) - change imported key to NULL if
     *      its primary key has been updated</li>
     *      <li>NuoDB::importedKeyNoAction (3) - do not allow update of primary
     *      key if it has been imported</li>
     *      <li>NuoDB::importedKeySetDefault (4) - change imported key to default
     *      values if its primary key has been updated</li>
     *      </ul></li>
     * <li>DELETE_RULE      short => What happens to the foreign key when primary
     *                      is deleted. Values are:
     *      <ul>
     *      <li>NuoDB::importedKeyCascade (0) - delete rows that import a deleted
     *      key</li>
     *      <li>NuoDB::importedKeySetNull (2) - change imported key to NULL if
     *      its primary key has been deleted</li>
     *      <li>NuoDB::importedKeyNoAction (3) - do not allow delete of primary
     *      key if it has been imported</li>
     *      <li>NuoDB::importedKeySetDefault (4) - change imported key to default
     *      if its primary key has been deleted</li>
     *      </ul></li>
     * <li>FK_NAME          const char* => foreign key name (may be null)</li>
     * <li>PK_NAME          const char* => primary key name (may be null)</li>
     * <li>DEFERRABILITY    short => can the evaluation of foreign key constraints
     *                      be deferred until commit. Values are:
     *      <ul>
     *      <li>NuoDB::importedKeyInitiallyDeferred (5) - see SQL92 for definition</li>
     *      <li>NuoDB::importedKeyInitiallyImmediate (6) - see SQL92 for definition</li>
     *      <li>NuoDB::importedKeyNotDeferrable (7) - see SQL92 for definition</li>
     *      </ul></li>
     * </ol>
     *
     * @param   catalog     provided for JDBC compatibility and ignored for %NuoDB
     * @param   schema      a schema name - must match the schema name as it is
     *                      stored in the database; "" retrieves those without a
     *                      schema; null means that the schema name should not be
     *                      used to narrow the search
     * @param   table       a table name - must match the table name as it is
     *                      stored in the database
     *
     * @return  ResultSet   with each row representing a primary key column
     *                      description.
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getImportedKeys(
                                const char* catalog,
                                const char* schema,
                                const char* table) = 0;

    /* not implemented */
    virtual ResultSet*      getExportedKeys(const char* catalog, const char* schema, const char* table) = 0;

    /* not implemented */
    virtual ResultSet*      getCrossReference(const char* parentCatalog, const char* parentSchema, const char* parentTable, const char* foreignCatalog, const char* foreignSchema, const char* foreignTable) = 0;

    /**
     * @brief   Retrieves a description of all the data types supported %NuoDB.
     *          They are ordered by DATA_TYPE and then by how closely the data
     *          type maps to the corresponding JDBC SQL type.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getTypeInfo().
     *
     * Each type description has the following columns:
     *
     * <ul>
     * <li>TYPE_NAME        const char* => Type name</li>
     * <li>DATA_TYPE        int => the type code defined by NuoDB::SqlType</li>
     * <li>PRECISION        int => maximum precision</li>
     * <li>LITERAL_PREFIX   const char* => prefix used to quote a literal (may
     *                      be null)</li>
     * <li>LITERAL_SUFFIX   const char* => suffix used to quote a literal (may
     *                      be null)</li>
     * <li>CREATE_PARAMS    const char* => parameters used in creating the type
     *                      (may be null)</li>
     * <li>NULLABLE         short => can you use NULL for this type. Values are:
     *      <ul>
     *      <li>NuoDB::typeNoNulls (0) - does not allow NULL values</li>
     *      <li>NuoDB::typeNullable (1) - allows NULL values</li>
     *      <li>NuoDB::typeNullableUnknown (2)- nullability unknown</li>
     *      </ul></li>
     * <li>CASE_SENSITIVE   boolean=> is it case sensitive.</li>
     * <li>SEARCHABLE       short => can you use "WHERE" based on this type.
     *                      Values are:
     *      <ul>
     *      <li>NuoDB::typePredNone (0) - No support</li>
     *      <li>NuoDB::typePredChar (1) - Only supported with WHERE .. LIKE</li>
     *      <li>NuoDB::typePredBasic (2) - Supported except for WHERE .. LIKE</li>
     *      <li>NuoDB::typeSearchable (3) - Supported for all WHERE ..</li>
     *      </ul></li>
     * <li>UNSIGNED_ATTRIBUTE   boolean => is it unsigned.</li>
     * <li>FIXED_PREC_SCALE     boolean => can it be a money value.</li>
     * <li>AUTO_INCREMENT       boolean => can it be used for an auto-increment value.</li>
     * <li>LOCAL_TYPE_NAME      const char* => localized version of type name (may be null)</li>
     * <li>MINIMUM_SCALE        short => minimum scale supported</li>
     * <li>MAXIMUM_SCALE        short => maximum scale supported</li>
     * <li>SQL_DATA_TYPE        int => unused</li>
     * <li>SQL_DATETIME_SUB     int => unused</li>
     * <li>NUM_PREC_RADIX       int => always 10 for %NuoDB</li>
     * </ol>
     *
     * @return  a ResultSet object in which each row is an SQL type description
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getTypeInfo() = 0;

    /**
     * @brief   Retrieves a description of the given table's indices and
     *          statistics. They are ordered by NON_UNIQUE, TYPE, INDEX_NAME,
     *          and ORDINAL_POSITION.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getIndexInfo().
     *
     * Each index column description has the following columns:
     *
     * <ol>
     * <li>TABLE_CAT        const char* => primary key table catalog (may be
     *                      null)<br /> Always set to empty string by %NuoDB</li>
     * <li>TABLE_SCHEM      const char* => table schema (may be null)</li>
     * <li>TABLE_NAME       const char* => table name</li>
     * <li>NON_UNIQUE       boolean => Can index values be non-unique.</li>
     * <li>INDEX_QUALIFIER  const char* => index catalog <br/>
     *                      Always set to empty string by %NuoDB</li>
     * <li>INDEX_NAME       const char* => index name; null when TYPE is
     *                      tableIndexStatistic</li>
     * <li>TYPE             short => index type. %NuoDB always sets this to
     *                      NuoDB::tableIndexOther (3)</li>
     * <li>ORDINAL_POSITION short => column sequence number within index</li>
     * <li>COLUMN_NAME      const char* => column name</li>
     * <li>ASC_OR_DESC      const char* => column sort sequence, %NuoDB always
     *                      sets this to NULL</li>
     * <li>CARDINALITY      int => Unused by %NuoDB and always sets to 0</li>
     * <li>PAGES            int => Unused by %NuoDB and always sets to 0</li>
     * <li>FILTER_CONDITION const char* => Unused by %NuoDB and always set to
     *                  empty string</li>
     * </ol>
     *
     * @param   catalog     provided for JDBC compatibility and ignored for %NuoDB
     * @param   schema      a schema name - must match the schema name as it is
     *                      stored in this database; "" retrieves those without
     *                      a schema; null means that the schema name should not
     *                      be used to narrow the search
     * @param   table       a table name - must match the table name as it is
     *                      stored in this database
     * @param   unique      when true, return only indices for unique values;
     *                      when false, return indices regardless of whether
     *                      unique or not
     * @param   approximate provided for JDBC compatibility and ignored for %NuoDB
     *
     * @return  ResultSet where each row is an index column description
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getIndexInfo(
                                const char* catalog,
                                const char* schema,
                                const char* table,
                                bool unique,
                                bool approximate) = 0;

    /* not implemented */
    virtual bool            supportsResultSetType(int type) = 0;

    /* not implemented */
    virtual bool            supportsResultSetConcurrency(int type, int concurrency) = 0;

    /* not implemented */
    virtual bool            ownUpdatesAreVisible(int type) = 0;

    /* not implemented */
    virtual bool            ownDeletesAreVisible(int type) = 0;

    /* not implemented */
    virtual bool            ownInsertsAreVisible(int type) = 0;

    /* not implemented */
    virtual bool            othersUpdatesAreVisible(int type) = 0;

    /* not implemented */
    virtual bool            othersDeletesAreVisible(int type) = 0;

    /* not implemented */
    virtual bool            othersInsertsAreVisible(int type) = 0;

    /* not implemented */
    virtual bool            updatesAreDetected(int type) = 0;

    /* not implemented */
    virtual bool            deletesAreDetected(int type) = 0;

    /* not implemented */
    virtual bool            insertsAreDetected(int type) = 0;

    /* not implemented */
    virtual bool            supportsBatchUpdates() = 0;

    /* not implemented */
    virtual ResultSet*      getUDTs(const char* catalog, const char* schemaPattern, const char* typeNamePattern, int* types) = 0;

    virtual Connection*     getConnection() = 0;

    /* not implemented */
    virtual bool            supportsSavepoints() = 0;

    /* not implemented */
    virtual bool            supportsNamedParameters() = 0;

    /* not implemented */
    virtual bool            supportsMultipleOpenResults() = 0;

    /* not implemented */
    virtual bool            supportsGetGeneratedKeys() = 0;

    /* not implemented */
    virtual ResultSet*      getSuperTypes(const char* catalog, const char* schemaPattern, const char* typeNamePattern) = 0;

    /* not implemented */
    virtual ResultSet*      getSuperTables(const char* catalog, const char* schemaPattern, const char* tableNamePattern) = 0;

    /* not implemented */
    virtual ResultSet*      getAttributes(const char* catalog, const char* schemaPattern, const char* typeNamePattern, const char* attributeNamePattern) = 0;

    /* not implemented */
    virtual bool            supportsResultSetHoldability(int holdability) = 0;

    /* not implemented */
    virtual int             getResultSetHoldability() = 0;

    /**
     * @brief   Retrieves the %NuoDB major release number
     *
     * @return  the %NuoDB major release number
     */
    virtual int             getDatabaseMajorVersion() = 0;

    /**
     * @brief   Retrieves the %NuoDB minor release number
     *
     * @return  the %NuoDB minor release number
     */
    virtual int             getDatabaseMinorVersion() = 0;

    /* not implemented */
    virtual int             getJDBCMajorVersion() = 0;

    /* not implemented */
    virtual int             getJDBCMinorVersion() = 0;

    /* not implemented */
    virtual int             getSQLStateType() = 0;

    /* not implemented */
    virtual bool            locatorsUpdateCopy() = 0;

    /* not implemented */
    virtual bool            supportsStatementPooling() = 0;

    /* not implemented */
    virtual RowIdLifetime*  getRowIdLifetime() = 0;

    /* c-api-suffix=with_catalog */
    virtual ResultSet*      getSchemas(const char* catalog, const char* schemaPattern) = 0;

    /* not implemented */
    virtual bool            supportsStoredFunctionsUsingCallSyntax() = 0;

    /* not implemented */
    virtual bool            autoCommitFailureClosesAllResultSets() = 0;

    /* not implemented */
    virtual ResultSet*      getClientInfoProperties() = 0;

    /**
     * @brief   Retrieves a description of the user functions available
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getFunctions(). Only user function descriptions matching
     * the schema and function name criteria are returned. They are ordered by
     * FUNCTION_SCHEM and FUNCTION_NAME.
     *
     * Each function description has the the following columns:
     *
     * <ol>
     * <li>FUNCTION_CAT     const char* => function catalog (may be null)<br />
     *                      ALways set to empty string by %NuoDB</li>
     * <li>FUNCTION_SCHEM   const char* => function schema (may be null)</li>
     * <li>FUNCTION_NAME    const char* => function name. This is the name used
     *                      to invoke the function</li>
     * <li>REMARKS          const char* => Always set to empty string in %NuoDB</li>
     * <li>FUNCTION_TYPE    short => kind of function. Values are:
     *      <ul>
     *      <li>NuoDB::functionNoTable (1) - Does not return a table</li>
     *      <li>NuoDB::functionReturnsTable (2)- Returns a table</li>
     *      </ul></li>
     * <li>SPECIFIC_NAME    const char* => The name which uniquely identifies
     *                      this function within its schema. For %NuoDB, this is
     *                      always the same as FUNCTION_NAME</li>
     * </ol>
     *
     * @param   catalog                 provided for JDBC compatibility and
     *                                  ignored for %NuoDB
     * @param   schemaPattern           a schema name pattern - must match the
     *                                  schema name as it is stored in the
     *                                  database; "" retrieves those without a
     *                                  schema; null means that the schema name
     *                                  should not be used to narrow the search
     * @param   functionNamePattern     a function name pattern - must match the
     *                                  function name as it is stored in the
     *                                  database
     *
     * @return  ResultSet with one row per matching function
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getFunctions(
                                const char* catalog,
                                const char* schemaPattern,
                                const char* functionNamePattern) = 0;

    /**
     * @brief   Retrieves a description of the database's user function
     *          parameters and return type.
     *
     * This method is written for compatibility with JDBC
     * %DatabaseMetaData#%getFunctionColumns().  Only descriptions matching the
     * schema, function and parameter name criteria are returned. The return
     * value, if any, is first. Next are the parameter descriptions in call
     * order. The column descriptions follow in column number order.
     *
     * Each row in the ResultSet is a parameter description, column description
     * or return type description with the following fields:
     *
     * <ol>
     * <li>FUNCTION_CAT     const char* => function catalog (may be null)<br />
     *                      Always set to empty string by %NuoDB</li>
     * <li>FUNCTION_SCHEM   const char* => function schema (may be null)</li>
     * <li>FUNCTION_NAME    const char* => function name</li>
     * <li>COLUMN_NAME      const char* => column/parameter name</li>
     * <li>COLUMN_TYPE      short => kind of column/parameter:
     *      <ul>
     *      <li>NuoDB::functionColumnUnknown (0) - nobody knows</li>
     *      <li>NuoDB::functionColumnIn (1) - IN parameter</li>
     *      <li>NuoDB::functionColumnInOut (2) - INOUT parameter</li>
     *      <li>NuoDB::functionColumnOut (3) - OUT parameter</li>
     *      <li>NuoDB::functionColumnReturn (4) - function return value</li>
     *      <li>NuoDB::functionColumnResult (5) - indicates that the parameter
     *      or column is a column in the ResultSet</li>
     *      </ul></li>
     * <li>DATA_TYPE        int => the type code defined by NuoDB::SqlType</li>
     * <li>TYPE_NAME        const char* => SQL type name</li>
     * <li>PRECISION        int => precision</li>
     * <li>LENGTH           int => length in bytes of data</li>
     * <li>SCALE            short => scale - null is returned for data types </li>
     *                      where SCALE is not applicable.</li>
     * <li>RADIX            short => %NuoDB always returns 0</li>
     * <li>NULLABLE         short => can it contain NULL, %NuoDB always returns 1
     *                      (NuoDB::functionNullable)</li>
     * <li>REMARKS          const char* => always empty string in %NuoDB</li>
     * <li>CHAR_OCTET_LENGTH int => the maximum length of binary and character
     *                      based columns. For any other datatype the returned
     *                      value is a NULL</li>
     * <li>ORDINAL_POSITION int => the ordinal position, starting from 1, for
     *                      the input and output parameters for a function. A
     *                      value of 0 is returned if this row describes the
     *                      function's return value. For result set columns, it
     *                      is the ordinal position of the column in the result
     *                      set starting from 1. If there are multiple result
     *                      sets, the column ordinal positions are implementation
     *                      defined.</li>
     * <li>IS_NULLABLE      const char* => ISO rules are used to determine the
     *                      nullability for a column.<br/>
     *      %NuoDB always returns an empty string, meaning the nullability for the
     *      column is unknown</li>
     * <li>SPECIFIC_NAME    const char* => the name which uniquely identifies
     *                      this function within its schema.<br />
     *      %NuoDB always returns the FUNCTION_NAME here</li>
     * </ol>
     *
     * @param   catalog                 provided for JDBC compatibility and
     *                                  ignored for %NuoDB
     * @param   schemaPattern           a schema name pattern - must match the
     *                                  schema name as it is stored in the
     *                                  database; "" retrieves those without a
     *                                  schema; null means that the schema name
     *                                  should not be used to narrow the search
     * @param   functionNamePattern     a function name pattern - must match the
     *                                  procedure name as it is stored in the
     *                                  database
     * @param   columnNamePattern       a column name pattern - must match the
     *                                  column name as it is stored in the database
     *
     * @return  ResultSet with each row representing a function parameter or column
     *
     * @throw   SQLException
     */
    virtual ResultSet*      getFunctionColumns(
                                const char* catalog,
                                const char* schemaPattern,
                                const char* functionNamePattern,
                                const char* columnNamePattern) = 0;

    /* not implemented */
    virtual ResultSet*      getPseudoColumns(const char* catalog, const char* schemaPattern, const char* tableNamePattern, const char* columnNamePattern) = 0;

    /* not implemented */
    virtual bool            generatedKeyAlwaysReturned() = 0;

    /* not implemented */
    virtual ResultSet*  getObjectPrivileges(const char* catalog, const char* schemaPattern, const char* namePattern, int objectType) = 0;

    /* not implemented */
    virtual ResultSet*  getUserRoles(const char* catalog, const char* schema, const char* rolePattern, const char* user) = 0;

    /* not implemented */
    virtual ResultSet*  getRoles(const char* catalog, const char* schema, const char* rolePattern) = 0;

    /* not implemented */
    virtual ResultSet*  getUsers(const char* catalog, const char* userPattern) = 0;

    /* not implemented */
    virtual int         objectVersion() = 0;

    /* not implemented */
    virtual bool        supportsStatementMetaData() = 0;

    /**
     * @brief   Retrieves a description of the triggers stored in the
     *          database
     *
     * Results are ordered by TABLE_SCHEM, TABLE_NAME and TRIGGER_NAME.
     *
     * Each trigger description has the the following columns:
     *
     * <ol>
     * <li>TABLE_CAT        const char* => table catalog (may be null)<br />
     *                      Always set to empty string by %NuoDB</li>
     * <li>TABLE_SCHEM      const char* => table schema (may be null)</li>
     * <li>TABLE_NAME       const char* => table name</li>
     * <li>TRIGGER_NAME     const char* => trigger name</li>
     * <li>CLASS_NAME       const char* => for Java triggers, the classid
     *                      specified in the CREATE TRIGGER command.</li>
     * <li>METHOD_NAME      const char* => for Java triggers, the fully qualified
     *                      name of the method to be invoked by the trigger.</li>
     * <li>POSITION         short => A user-provided index that defines the
     *                      order of invocations of triggers (lower value means
     *                      early execution). This corresponds to the POSITION
     *                      number argument for CREATE TRIGGER</li>
     * <li>TYPE_MASK        int => Defines the trigger event:
     *      <ul>
     *      <li>1 = Before Insert</li>
     *      <li>2 = After Insert</li>
     *      <li>4 = Before Update</li>
     *      <li>8 = After Update</li>
     *      <li>16 = Before Delete</li>
     *      <li>32 = After Delete</li>
     *      <li>64 = Before Commit - reserved for future use</li>
     *      <li>128 = After Commit - reserved for future use</li>
     *      </ul></li>
     * <li>ACTIVE           short => Defined as: 1=Active, 0=Inactive</li>
     * </ol>
     *
     * @param   catalog                 provided for JDBC compatibility and
     *                                  ignored for %NuoDB
     * @param   schemaPattern           a schema name pattern - must match the
     *                                  schema name as it is stored in the
     *                                  database; "" retrieves those without a
     *                                  schema; null means that the schema name
     *                                  should not be used to narrow the search
     * @param   tableNamePattern        a table name pattern - must match the
     *                                  table name as it is stored in the
     *                                  database
     * @param   triggerNamePattern      a trigger name pattern - must match the
     *                                  trigger name as it is stored in the
     *                                  database
     *
     * @return  ResultSet with each row representing a trigger
     *
     * @throw   SQLException
     */
    virtual ResultSet*  getTriggers(
                            const char* catalog,
                            const char* schemaPattern,
                            const char* tableNamePattern,
                            const char* triggerNamePattern) = 0;
};
}                   // end namespace NuoDB
