/* Copyright 2013-2015 NuoDB, Inc. All rights reserved */

#pragma once

#include <stdint.h>

namespace NuoDB
{
static const int STATEMENTMETADATA_VERSION  = 1;

/**
 * @brief   A class that can be used to get information about the types and 
 *          properties for each parameter marker in a CallableStatement object. 
 */
class ParameterMetaData
{
protected:
    virtual ~ParameterMetaData() {}

public:
    /**
     * @brief   Retrieves the number of parameters in the CallableStatement object 
     *          for which this ParameterMetaData object contains information.
     *
     * @return  the number of parameters
     */
    virtual int     getParameterCount() = 0;

    /**
     * @brief   Retrieves the designated parameter's SQL type.
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     *
     * @param   sqlType         the type code defined by NuoDB::SqlType. If the 
     *                          parameter is of type NUODB_NUMERIC or NUODB_DECIMAL, 
     * @return  the parameter's SQL type as defined in NuoDB::SqlType.
     */
    virtual int     getParameterType(int index) = 0;

    /** 
     * @brief   Retrieves the designated parameter's specified column size.
     *
     * The returned value represents the maximum column size for the given 
     * parameter. For numeric data, this is the maximum precision. For character 
     * data, this is the length in characters. For datetime datatypes, this is the 
     * length in characters of the String representation (assuming the maximum 
     * allowed precision of the fractional seconds component). For binary data, 
     * this is the length in bytes. 
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     *
     * @return  the precision for the given parameter
     */
    virtual int     getPrecision(int index) = 0;

    /** 
     * @brief   Retrieves the designated parameter's specified scale
     *
     * Retrieves the designated parameter's number of digits to right of the 
     * decimal point. 0 is returned for data types where the scale is not applicable.
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     *
     * @return  the scale for the given parameter or 0 if scale is not applicable
     */
    virtual int     getScale(int index) = 0;

    /** 
     * @brief   Determine if the designated parameter is nullable
     *
     * Retrieves whether null values are allowed in the designated parameter.
     *
     * @param   index   the first parameter is 1, the second is 2, ...
     *
     * @return  true if the given parameter allows NULL values
     */
    virtual bool    isNullable(int index) = 0;

    /* not implemented */
    virtual int     objectVersion() = 0;
};
}                   // end namespace NuoDB
