/* Copyright 2013-15 NuoDB, Inc. All rights reserved */
#pragma once

#include <stdint.h>
#include "SqlExports.h"

namespace NuoDB
{
class Statement;
class ResultSetMetaData;
class BigDecimal;
class Properties;
class Timestamp;
class Clob;
class Blob;
class Date;
class Time;
class RowId;
class SQLWarning;
struct Bytes;

static const int    RESULTSET_VERSION           = 1;
static const int    FETCH_FORWARD               = 1000;
static const int    FETCH_REVERSE               = 1001;
static const int    FETCH_UNKNOWN               = 1002;
static const int    TYPE_FORWARD_ONLY           = 1003;
static const int    TYPE_SCROLL_INSENSITIVE     = 1004;
static const int    TYPE_SCROLL_SENSITIVE       = 1005;
static const int    CONCUR_READ_ONLY            = 1007;
static const int    CONCUR_UPDATABLE            = 1008;
static const int    HOLD_CURSORS_OVER_COMMIT    = 1;
static const int    CLOSE_CURSORS_AT_COMMIT     = 2;

/**
 * @brief   A table of data representing a database result set, which is usually
 *          generated by executing a statement that queries the database.
 *
 * A ResultSet object maintains a cursor pointing to its current row of data.
 * Initially the cursor is positioned before the first row. The next() method
 * moves the cursor to the next row, and because it returns false when there
 * are no more rows in the ResultSet object, it can be used in a while loop to
 * iterate through the result set.
 *
 * A ResultSet object is not updatable and has a cursor that moves forward only.
 * Thus, you can iterate through it only once and only from the first row to the
 * last row.
 *
 * The ResultSet class provides getter methods (getBoolean, getLong, and so on)
 * for retrieving column values from the current row. Values can be retrieved
 * using either the index number of the column or the name of the column. In
 * general, using the column index will be more efficient. Columns are numbered
 * from 1. Result set columns within each row should be read in left-to-right
 * order, and each column should be read only once.
 *
 * For the getter methods, a %NuoDB C++ driver attempts to convert the underlying
 * SQL data to the C++ type specified in the getter method and returns a suitable
 * C++ value.
 *
 * Column names used as input to getter methods are case insensitive. The column
 * name option is designed to be used when column names are used in the SQL query
 * that generated the result set. For columns that are NOT explicitly named in
 * the query, it is best to use column numbers. If column names are used, the
 * programmer should take care to guarantee that they uniquely refer to the
 * intended columns, which can be assured with the SQL AS clause.
 *
 * A ResultSet object is automatically closed when the Statement object that
 * generated it is closed or re-executed.
 *
 * The number, types and properties of a ResultSet object's columns are provided
 * by the ResultSetMetaData object returned by the ResultSet.getMetaData() method.
 */
class ResultSet
{

protected:
    virtual ~ResultSet() {}

public:
    /**
     * @brief   Increment the reference count for this object
     */
    virtual void    addRef() = 0;

    /**
     * @brief   Decrement the reference count for this object and if the 
     *          reference count is 0, delete the object
     *
     * @return  the reference count for this object; 0 if the object was deleted
     */
    virtual int     release() = 0;

    /**
     * @brief   Releases this ResultSet object's database resources and calls its 
     *          destructor immediately instead of waiting for this to happen when 
     *          it is automatically closed.
     *
     * The closing of a ResultSet object does not close the Blob or Clob objects 
     * created by the ResultSet. Blob or Clob objects remain valid for at least 
     * the duration of the transaction in which they are creataed, unless their 
     * destructor is invoked.
     *
     * When a ResultSet is closed, any ResultSetMetaData instances that were 
     * created by calling the getMetaData() method remain accessible.
     *
     * Note: A ResultSet object is automatically closed by the Statement object 
     * that generated it when that Statement object is closed or re-executed.
     *
     * Calling the method close() on a ResultSet object that is already closed 
     * is a no-op.
     *
     * @throw   SQLException
     */
    virtual void    close() = 0;

    /**
     * @brief   Moves the cursor froward one row from its current position.
     *
     * A ResultSet cursor is initially positioned before the first row; the
     * first call to the method next() makes the first row the current row; the
     * second call makes the second row the current row, and so on.
     *
     * When a call to the next() method returns false, the cursor is positioned
     * after the last row. Any invocation of a ResultSet method which requires a
     * current row will result in a SQLException being thrown.
     *
     * @return  true if the new current row is valid and false if there are
     *          no more rows
     *
     * @throw   SQLException
     */
    virtual bool    next() = 0;

    /**
     * @brief   Reports whether the last column read had a value of SQL NULL.
     *
     * Note that you must first call one of the getter methods on a column to
     * try to read its value and then call the method wasNull() to see if the
     * value read was SQL NULL.
     *
     * This method is used to differentiate a zero value from NULL.  So getInt()
     * can return 0 but it is really SQL NULL.
     *
     * @return  true if the last column value read was SQL NULL and false otherwise
     *
     * @throw   SQLException
     */
    virtual bool    wasNull() = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a const char*
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual const char* getString(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a bool
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is false
     *
     * @throw   SQLException
     */
    virtual bool    getBoolean(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a char
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    virtual char    getByte(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a short
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    virtual short   getShort(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a 32-bit integer
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    virtual int getInt(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a 64-bit integer
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    virtual int64_t getLong(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a float
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    virtual float   getFloat(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a double
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    virtual double      getDouble(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a BigDecimal object
     *
     * @param   columnIndex     the first column is 1, the second is 2, ...
     * @param   scale           the number of digits to the right of the decimal point
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=with_scale */
    virtual BigDecimal* getBigDecimal(int columnIndex, int scale) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a Bytes object
     *
     * The memory used by the Bytes object returned by getBytes() is managed by 
     * the driver and the callerd does not need to do so.
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual Bytes       getBytes(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Date object
     *
     * Note: The user is responsible for freeing the resources allocated in the 
     * returned Date object by calling Date::release().
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual Date*       getDate(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Time object
     *
     * Note: The user is responsible for freeing the resources allocated in the 
     * returned Time object by calling Time::release().
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual Time*       getTime(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Timestamp object
     *
     * Note: The user is responsible for freeing the resources allocated in the 
     * returned Timestamp object by calling Timestamp::release().
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual Timestamp*  getTimestamp(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a const char*
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual const char* getString(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a bool
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is false
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual bool        getBoolean(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a char
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual char        getByte(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a short
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual short       getShort(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a 32-bit integer
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual int         getInt(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a 64-bit integer
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual int64_t     getLong(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a float
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual float       getFloat(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a double
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is 0
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual double      getDouble(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a BigDecimal object
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     * @param   scale           the number of digits to the right of the decimal point
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label_with_scale */
    virtual BigDecimal* getBigDecimal(const char* columnLabel, int scale) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a Bytes object
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual Bytes       getBytes(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Date object
     *
     * Note: The user is responsible for freeing the resources allocated in the 
     * returned Date object by calling Date::release().
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual Date*       getDate(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Time object
     *
     * Note: The user is responsible for freeing the resources allocated in the 
     * returned Time object by calling Time::release().
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual Time*       getTime(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Timestamp object
     *
     * Note: The user is responsible for freeing the resources allocated in the 
     * returned Timestamp object by calling Timestamp::release().
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual Timestamp*  getTimestamp(const char* columnLabel) = 0;

    /* not implemented */
    virtual SQLWarning* getWarnings() = 0;

    /* not implemented */
    virtual void        clearWarnings() = 0;

    /* not implemented */
    virtual const char* getCursorName() = 0;

    /**
     * @brief   Retrieves the number, types and properties of this ResultSet 
     *          object's columns.
     *
     * @return  a pointer to a ResultSetMetaData object
     *
     * @throw   SQLException
     */
    virtual ResultSetMetaData*  getMetaData() = 0;

    /**
     * @brief   Maps the given ResultSet column label to its ResultSet column index
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     * 
     * @return  the column index of the given column name
     *
     * @throw   SQLException
     */
    virtual int                 findColumn(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a BigDecimal object
     *
     * @param   columnIndex     the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual BigDecimal*         getBigDecimal(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a BigDecimal object
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual BigDecimal*         getBigDecimal(const char* columnLabel) = 0;

    /* begin not implemented ... */
    virtual bool                isBeforeFirst() = 0;
    virtual bool                isAfterLast() = 0;
    virtual bool                isFirst() = 0;
    virtual bool                isLast() = 0;
    virtual void                beforeFirst() = 0;
    virtual void                afterLast() = 0;
    virtual bool                first() = 0;
    virtual bool                last() = 0;
    virtual int                 getRow() = 0;
    virtual bool                absolute(int row) = 0;
    virtual bool                relative(int rows) = 0;
    virtual bool                previous() = 0;
    virtual void                setFetchDirection(int direction) = 0;
    virtual int                 getFetchDirection() = 0;
    virtual void                setFetchSize(int rows) = 0;
    virtual int                 getFetchSize() = 0;
    virtual int                 getType() = 0;
    virtual int                 getConcurrency() = 0;
    virtual bool                rowUpdated() = 0;
    virtual bool                rowInserted() = 0;
    virtual bool                rowDeleted() = 0;
    virtual void                updateNull(int columnIndex) = 0;
    virtual void                updateBoolean(int columnIndex, bool x) = 0;
    virtual void                updateByte(int columnIndex, char x) = 0;
    virtual void                updateShort(int columnIndex, short x) = 0;
    virtual void                updateInt(int columnIndex, int x) = 0;
    virtual void                updateLong(int columnIndex, int64_t x) = 0;
    virtual void                updateFloat(int columnIndex, float x) = 0;
    virtual void                updateDouble(int columnIndex, double x) = 0;
    virtual void                updateBigDecimal(int columnIndex, BigDecimal* x) = 0;
    virtual void                updateString(int columnIndex, const char* x) = 0;
    virtual void                updateBytes(int columnIndex, Bytes x) = 0;
    virtual void                updateDate(int columnIndex, Date* x) = 0;
    virtual void                updateTime(int columnIndex, Time* x) = 0;
    virtual void                updateTimestamp(int columnIndex, Timestamp* x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateNull(const char* columnLabel) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateBoolean(const char* columnLabel, bool x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateByte(const char* columnLabel, char x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateShort(const char* columnLabel, short x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateInt(const char* columnLabel, int x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateLong(const char* columnLabel, int64_t x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateFloat(const char* columnLabel, float x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateDouble(const char* columnLabel, double x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateBigDecimal(const char* columnLabel, BigDecimal* x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateString(const char* columnLabel, const char* x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateBytes(const char* columnLabel, Bytes x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateDate(const char* columnLabel, Date* x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateTime(const char* columnLabel, Time* x) = 0;

    /* c-api-suffix=by_label */
    virtual void                updateTimestamp(const char* columnLabel, Timestamp* x) = 0;
    virtual void                insertRow() = 0;
    virtual void                updateRow() = 0;
    virtual void                deleteRow() = 0;
    virtual void                refreshRow() = 0;
    virtual void                cancelRowUpdates() = 0;
    virtual void                moveToInsertRow() = 0;
    virtual void                moveToCurrentRow() = 0;
    virtual Statement*          getStatement() = 0;
    /* ... end not implemented */

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Blob object
     * 
     * The Blob returned is valid only as long as the current row is active in 
     * the ResultSet. If an application needs to reference the Blob after calling
     * next() on the ResultSet, it should call addRef() on the Blob.
     * The caller does not need call release() on the Blob object returned by 
     * getBlob().
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual Blob* getBlob(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Clob object
     * 
     * The Clob returned is valid only as long as the current row is active in 
     * the ResultSet. If an application needs to reference the Clob after calling
     * next() on the ResultSet, it should call addRef() on the Clob.
     *
     * @param   columnIndex      the first column is 1, the second is 2, ...
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    virtual Clob*   getClob(int columnIndex) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Blob object
     * 
     * The Blob returned is valid only as long as the current row is active in 
     * the ResultSet. If an application needs to reference the Blob after calling
     * next() on the ResultSet, it should call addRef() on the Blob.
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual Blob*   getBlob(const char* columnLabel) = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a pointer to a Clob object
     * 
     * The Clob returned is valid only as long as the current row is active in 
     * the ResultSet. If an application needs to reference the Clob after calling
     * next() on the ResultSet, it should call addRef() on the Clob.
     *
     * @param   columnLabel the label for the column specified with the SQL AS 
     *                      clause. If the SQL AS clause was not specified, then
     *                      the label is the name of the column
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=by_label */
    virtual Clob*   getClob(const char* columnLabel) = 0;

    /* not implemented */
    virtual void    updateBlob(int columnIndex, Blob* x) = 0;

    /* not implemented */
    /* c-api-suffix=by_label */
    virtual void    updateBlob(const char* columnLabel, Blob* x) = 0;

    /* not implemented */
    virtual void    updateClob(int columnIndex, Clob* x) = 0;

    /* c-api-suffix=by_label */
    virtual void    updateClob(const char* columnLabel, Clob* x) = 0;

    /* not implemented */
    virtual RowId*  getRowId(int columnIndex) = 0;

    /* not implemented */
    /* c-api-suffix=by_label */
    virtual RowId*  getRowId(const char* columnLabel) = 0;

    /* not implemented */
    virtual void    updateRowId(int columnIndex, RowId* x) = 0;

    /* not implemented */
    /* c-api-suffix=by_label */
    virtual void    updateRowId(const char* columnLabel, RowId* x) = 0;

    /* not implemented */
    virtual int     getHoldability() = 0;

    /* not implemented */
    virtual bool    isClosed() = 0;

    /* not implemented */
    virtual int     objectVersion() = 0;

    /**
     * @brief   Retrieves the value of the designated column in the current row 
     *          of this ResultSet object as a const char*
     *
     * @param   columnIndex     the first column is 1, the second is 2, ...
     * @param   n_chars         only provide the first n_chars of the column value
     *
     * @return  the column value; if the value is SQL NULL, the value returned is null
     *
     * @throw   SQLException
     */
    /* c-api-suffix=and_length */
    virtual const char* getString(int columnIndex, int* n_chars) = 0;
};
}                   // end namespace NuoDB

#define CHECK_COLUMN_BOUNDS(COL_IDX, COL_NUM, ARG_TYPE, CHECKED_FN) \
    { \
        if (COL_IDX < 1 || COL_IDX > COL_NUM) { \
            throw SQLError(RUNTIME_ERROR, "invalid " #ARG_TYPE " number (%d) for " #CHECKED_FN, COL_IDX); } \
    }
