// (C) Copyright 2013-2019 NuoDB, Inc.  All Rights Reserved.

#pragma once

#include <exception>
#include <stddef.h>

#include "SQLExceptionConstants.h"

#if __cpluplus >= 201103L
 #define NUODB_NOEXCEPT_FINAL noexcept final 
#else
 #define NUODB_NOEXCEPT_FINAL throw() 
#endif

using namespace NuoDB::NuoDBSqlConstants;

namespace NuoDB
{

/**
 * @brief An exception that provides information on a database access error or
 * other errors.
 *
 * SQLException is the only type of exception thrown by the %NuoDB C++ driver.
 * It provides the client with at least the SQL code that caused the exception
 * which corresponds to those defined in NuoDB::NuoDBSqlConstants::SqlCode, and
 * the text of the error message.  In some cases it also provides a stack trace
 * and the SQL state.
 */

class SQLException : public std::exception
{
public:
    /**
     * @brief   Get the SQL code set by the Exception that is being thrown.
     *          This corresponds to those defined in NuoDB::NuoDBSqlConstants.
     *
     * @return  The SQL code
     */
    virtual int getSqlcode() const = 0;

    /**
     * @brief   Get the text of the Exception that is being thrown
     *
     * @return  The error message
     */
    virtual const char* getText() const = 0;

    /**
     * @brief   Get a stack trace for the SQL Exception being thrown
     *
     * @param   buffer  pointer to a char* buffer to which the stack trace will
     *                  be written. The caller is responsible for allocating
     *                  and deleting this buffer.
     *
     * @return  A pointer to the filled in buffer
     */
    virtual char* getTrace(char* buffer, size_t len) const = 0;

    /**
     * @brief   Get the SQL state as defined by the SQL standard; this may or
     *          may not be set by the exception
     *
     * @return  The SQL state, or empty string if the SQL state is unavailable
     */
    virtual const char* getSQLState() const = 0;

    virtual const char* getSQLStackTrace() const = 0;

    virtual const char* what() const NUODB_NOEXCEPT_FINAL { return getText(); }
};
}
