/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_H_
#define _NUODB_H_ 1

/**
 * @file NuoDB.h
 *
 * @brief NuoDB C-Language API.
 *
 * The is the main header file for the NuoDB C-Language API.  This
 * header file includes all of the other header files, so client
 * applications only need to #include nuodb/NuoDB.h.
 */


/*! @mainpage NuoDB C Driver API
 *
 * The NuoDB C driver provides a C language API for connecting to
 * NuoDB Database and executing SQL statements. For more information,
 * see:
 * <a href="http://doc.nuodb.com/Latest/Default.htm#Using-the-NuoDB-C-Driver.htm"><strong>Client Interfaces... C</strong></a>
 * in the <a href="http://doc.nuodb.com/Latest/Default.htm#Client-Development.htm"><strong>Client Interfaces</strong></a>
 * section of the %NuoDB Documentation.  There is a C driver example in
 * that documentation.
 *
 * The NuoDB C Driver API contains C-Language data structures that
 * represent functional types.  The functional types are object
 * oriented because they contain separate "state" and "methods".  All
 * functional types have an internal data field named "_state", which
 * should not be used by client applications.  The functional types
 * also have data fields that are function pointers, which are the
 * methods provided by that functional type.  All functional methods
 * take, as their first argument, a "this" pointer.  Which is similar
 * to the "this" pointer in C++ member function.  Clients are expected
 * to pass the functional type's current instance as the "this"
 * pointer.
 *
 * The following is a list of the functional types:
 *
 * @li NuoDB_Connection
 * @li NuoDB_DatabaseMetaData
 * @li NuoDB_Error
 * @li NuoDB_Lob
 * @li NuoDB_Options
 * @li NuoDB_ResultSet
 * @li NuoDB_ResultSetMetaData
 * @li NuoDB_Statement
 * @li NuoDB_Temporal
 *
 * There are global factory functions for creating some of those
 * functional types.  The factory functions are named in the format:
 * <functional-type>_Create().  Calling one of the *_Create()
 * functions will construct an initialized functional type.
 *
 * There are global "Free" functions that are used to release all
 * resources held by a functional type. The free functions are named
 * in the format: <functional-type>_Free().  The client application is
 * expected to free functional objects when it is finished using them.
 */

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
} /* extern "C" { */
#endif

typedef unsigned char   nuodb_bool_t;
#define NUODB_TRUE  1
#define NUODB_FALSE 0

#if defined(NUODB_CAPI_EXPORT) && defined(NUODB_CAPI_STATIC)
# error Both NUODB_CAPI_EXPORT and NUODB_CAPI_STATIC defined in the same translation unit
#endif

#if defined(_WIN32)
# define NUODB_CAPI_STDCALL __stdcall
# if defined(NUODB_CAPI_EXPORT)
#  define NUODB_CAPI __declspec(dllexport)
# elif defined(NUODB_CAPI_STATIC)
#  define NUODB_CAPI
# else
#  define NUODB_CAPI __declspec(dllimport)
# endif
#else
# define NUODB_CAPI_STDCALL
# if __GNUC__ >= 4
#  define NUODB_CAPI __attribute__((visibility("default")))
# else
#  define NUODB_CAPI
# endif
#endif

#define NUODB_CAPICALL NUODB_CAPI_STDCALL

#include <nuodb/structs.h>
#include <nuodb/error.h>
#include <nuodb/utils.h>
#include <nuodb/options.h>
#include <nuodb/temporal.h>
#include <nuodb/lob.h>
#include <nuodb/resultSet.h>
#include <nuodb/resultSetMetaData.h>
#include <nuodb/statement.h>
#include <nuodb/connection.h>
#include <nuodb/databaseMetaData.h>

#define NUODB_CDRIVER_MAJOR_VERSION 2
#define NUODB_CDRIVER_MINOR_VERSION 6

#endif /* end of #ifndef _NUODB_H_ */
