/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_ERROR_H_
#define _NUODB_ERROR_H_ 1

/**
 * @file error.h
 * @brief NuoDB Error information.
 */

/*! Enum defining status codes used by the C-Language API. */
enum NuoDB_Status_enum {
    NUODB_NO_DATA = 1,   /*!< indicates when is is no more data */
    NUODB_SUCCESS = 0,   /*!< success */
    NUODB_SYNTAX_ERROR = -1, /*!< SQL syntax error, Example: "wrong number of arguments for function" */
    NUODB_FEATURE_NOT_YET_IMPLEMENTED = -2, /*!< Feature is not yet implemented */
    NUODB_BUG_CHECK = -3,  /*!< An internal database error has occurred */
    NUODB_COMPILE_ERROR = -4, /*!< An error has occurred during compilation of the SQL statement,
                                 Example: "inconsistent select list in union branches" */
    NUODB_RUNTIME_ERROR = -5, /*!< An error has occurred during execution of an SQL statement,
                                 Example: "ResultSet has been closed" */
    NUODB_OCS_ERROR = -6, /*!< Unused */
    NUODB_NETWORK_ERROR = -7, /*!< A network communication has occurred, either with the Broker
                                 or TE, Example: "no nodes are available for database" */
    NUODB_CONVERSION_ERROR = -8, /*!< An conversion error has occurred during execution of an SQL
                                    statement, Example: "Numeric overflow converting..." */
    NUODB_TRUNCATION_ERROR = -9, /*!< A truncation error has occurred during execution of an SQL
                                    statement, Example: "Assignment of n bytes into type of maximum
                                    length m" */
    NUODB_CONNECTION_ERROR = -10, /*!< Error related to making an SQL connection or to an already
                                         existing SQL connection, Examples: "database already open",
                                         "bad statement handle" */
    NUODB_DDL_ERROR = -11, /*!< A runtime error has occurred during execution of an SQL DDL
                              statement, Example: "table already defined" */
    NUODB_APPLICATION_ERROR = -12, /*!< An application specific error has occurred, Example: "Cannot
                                      execute PreparedStatement with SQL parameter"*/
    NUODB_SECURITY_ERROR = -13, /*!< A database security error has occurred, such as bad credentials,
                                   Examples: "no username specified", "not a known user", "requested
                                   access to foo is denied", "Unsupported cipher requested"*/
    NUODB_DATABASE_CORRUPTION = -14, /*!< Unused */
    NUODB_VERSION_ERROR = -15, /*!< Unused */
    NUODB_LICENSE_ERROR = -16, /*!< Unused */
    NUODB_INTERNAL_ERROR = -17, /*!< An internal error has occurred. This should never happen.
                                   Examples: "function hasn't been compiled", "Unexpected NULL lower
                                   bound node" */
    NUODB_DEBUG_ERROR = -18, /*!< Unused */
    NUODB_LOST_BLOB = -19, /*!< Unused */
    NUODB_INCONSISTENT_BLOB = -20, /*!< Unused */
    NUODB_DELETED_BLOB = -21, /*!< Unused */
    NUODB_LOG_ERROR = -22, /*!< Unused */
    NUODB_DATABASE_DAMAGED = -23, /*!< Unused */
    NUODB_UPDATE_CONFLICT = -24, /*!< Server error has occurred, Example: "pending update
                                    rejected, transaction nnn" */
    NUODB_NO_SUCH_TABLE = -25, /*!< An invalid table reference has occurred, Examples: "Table has
                                  been dropped", "Can't find table" */
    NUODB_INDEX_OVERFLOW = -26, /*!< Unused */
    NUODB_UNIQUE_DUPLICATE = -27, /*!< Server error regarding duplicate values in a unique index" */
    NUODB_UNCOMMITTED_UPDATES = -28,
    NUODB_DEADLOCK = -29, /*!< Server deadlock error */
    NUODB_OUT_OF_MEMORY_ERROR = -30, /*!< Unused */
    NUODB_OUT_OF_RECORD_MEMORY_ERROR = -31, /*!< Unused */
    NUODB_LOCK_TIMEOUT = -32, /*!< Unused */
    NUODB_PLATFORM_ERROR = -36, /*!< Generic server error */
    NUODB_NO_SCHEMA = -37, /*!< An error occurred because no schema specified where schema is required. */
    NUODB_CONFIGURATION_ERROR = -38, /*!< Server configuration error, Example: "Could not find a
                                        valid archive at the given location" */
    NUODB_READ_ONLY_ERROR = -39, /*!< An error occurred where an attempt was made to write to a read
                                    only connection. */
    NUODB_NO_GENERATED_KEYS = -40, /*!< Unused */
    NUODB_THROWN_EXCEPTION = -41, /*!< This is used by the SQL THROW control statement */
    NUODB_INVALID_TRANSACTION_ISOLATION = -42, /*!< Unused */
    NUODB_UNSUPPORTED_TRANSACTION_ISOLATION = -43, /*!< An error occurred where the transaction level specified for
                                                      connection is unsupported */
    NUODB_INVALID_UTF8 = -44, /*!< An error occurred where a string that is expected to be UTF-8
                                 is not */
    NUODB_CONSTRAINT_ERROR = -45, /*!< A SQL constraint violation has occurred */
    NUODB_UPDATE_ERROR = -46, /*!< An attempt was made to update a database object that cannot be
                                 updated, Example: "Table returned by a stored procedure cannot be
                                 manipulated" */
    NUODB_I18N_ERROR = -47, /*!< An error having to do with Internationalization has occurred,
                               Example: "Error getting default converter" */
    NUODB_OPERATION_KILLED = -48, /*!< A SQL operation was killed, perhaps by the KILL STATEMENT
                                     command */
    NUODB_INVALID_STATEMENT = -49, /*!< An invalid reference to a SQL statement occurred, Example: "On
                                      connection n, Unable to kill statement m. No such statement id" */
    NUODB_IS_SHUTDOWN = -50, /*!< An error occurred because the database is shutting down. */
    NUODB_IN_QUOTED_STRING = -51, /*!< An internal error generated by parser when the statement string
                                     ends within a quoted string = used by nuosql, */
    NUODB_BATCH_UPDATE_ERROR = -52, /*!< Catchall for Batch update exceptions */
    NUODB_JAVA_ERROR = -53, /*!< An error has occurred, related to Java stored procedures */
    NUODB_INVALID_FIELD = -54, /*!< A server error setting a value to an invalid field */
    NUODB_INVALID_INDEX_NULL = -55, /*!< An invalid null was inserted into index */
    NUODB_INVALID_OPERATION = -56, /*!< A generic error related to an invalid operation, Examples:
                                      "Maximum number of open result sets exceeded", "Maximum number of
                                      open statements exceeded" */
    NUODB_INVALID_STATISTICS = -57, /*!< Index statistics for a specified query have become invalid */
    NUODB_INVALID_GENERATOR = -58, /*!< Badly defined generator defined in older version of nuodb. Need to
                                ALTER TABLE to change the columns type to either INT, BIGINT, NUMERIC or STRING */
    NUODB_OPERATION_TIMEOUT = -59, /*!< Query timeout period expired before a statement object executed.
                                Examples: Timeout of n micro second(s) exceeded", "Timeout of n second(s) exceeded" */
    NUODB_NO_SUCH_INDEX = -60, /*!< An invalid index reference has occurred, Examples: "Index has
                                    been dropped", "Can't index table" */
    NUODB_NO_SUCH_SEQUENCE = -61, /*!< An invalid sequence reference has occurred, Examples: "Sequence has
                                   been dropped", "Sequence doesn't exist" */
    NUODB_XAER_PROTO = -62, /*!< An error with the XA protocol has occurred, Examples:
                                 * "TransactionId already prepared" */
    NUODB_UNKNOWN_ERROR = -63, /*!*< An unknown error code */
    NUODB_TRANSACTIONAL_LOCK_ERROR = -64, /*!*< An error occurred while acquiring a transactionl lock */
    NUODB_FINAL_ENUM_MEMBER_IS_UNUSED = -65,
};

/*! Status codes used by the C-Language API. */
typedef enum NuoDB_Status_enum  NuoDB_Status;

#ifdef __cplusplus
extern "C" {
#endif

/*! %NuoDB error information data structure
 *
 * The NuoDB_Error data structure contains information about a error
 * condition.
 *
 * There are three mechanisms for error handling:
 * @li Global error handling through callback functions provided by
 * the client, registered with the function
 * NuoDB_Error_registerCallback().
 * @li Local error handling by checking integer return codes from functions.
 * Functions that return a error code will be integer zero (0) to indicate
 * success, or a negative integer to indicate an error condition.
 * @li Contextual thread error checking by calling the function
 * NuoDB_Error_getLastErrorInfo() to retrive the last known error.
 */
struct NuoDB_Error
{
    /** @internal */
    _NuoDB_Error_state_t* _state;

    /**
     * @brief   Returns the error status, which is a enumerated
     * integer code for errors that can be returned by this driver.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  NuoDB_Status enumerated type
     */
    NuoDB_Status (* getStatus)(NuoDB_Error* _this);

    /**
     * @brief   Returns the NuoDB_Connection for the specified error
     * data structure.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  pointer to NuoDB_Connection.
     */
    NuoDB_Connection*   (* getConnection)(NuoDB_Error* _this);

    /**
     * @brief   Returns the NuoDB_Statement for the specified error
     * data structure, if known.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  pointer to the NuoDB_Statement data structure that was
     * involved with the error.  If the NuoDB_Statement was unknown at
     * the time of the error, this function will return NULL.
     */
    NuoDB_Statement*    (* getStatement)(NuoDB_Error* _this);

    /**
     * @brief   Returns text of the error message.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  a pointer to a string that contains the error message.
     */
    const char*         (* getText)(NuoDB_Error* _this);

    /**
     * @brief   Returns stack trace for the error.
     *
     * If the stack trace is known at the point of the error
     * condition, then the return value will contain a pointer to
     * a string that represents the entire stack trace.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  pointer to a string that contains the stack trace, or
     * NULL if the stack trace is unknown at the time of the error.
     */
    const char*         (* getTrace)(NuoDB_Error* _this);

    /**
     * @brief   Returns the SQL State string.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  a pointer to a string that contains the SQL State code.
     */
    const char*         (* getSqlState)(NuoDB_Error* _this);
};

/**
 * @brief   Obtain the last known error.
 *
 * This function returns a pointer to the last known NuoDB_Error that
 * occurred in the context of the caller's thread.  The NuoDB_Error
 * data structure can be used to obtain the last known error code,
 * error text, stack trace, and SQL State.
 */
NUODB_CAPI NuoDB_Error*  NUODB_CAPICALL NuoDB_Error_getLastErrorInfo();

/**
 * @brief   Clear the last known error.
 *
 * This function clears the last known NuoDB_Error for the caller's thread.
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Error_clearLastErrorInfo();

/**
 * @brief   Register an error callback function.
 *
 * This function allows the client application to register a function
 * that will be called in the event of a error.  The registered
 * function is specific to the caller's thread.  When an error occurs,
 * the C driver will call the error function on the caller's thread.
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Error_registerCallback(NuoDB_Status (* error_callback)(NuoDB_Error* error));

/**
 * @internal
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Error_setSqlState(const char* sqlState);

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* end of #ifndef _NUODB_ERROR_H_  */
