/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_LOB_H_
#define _NUODB_LOB_H_ 1

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file lob.h
 * @brief NuoDB Large Object.
 */

/*! Enum defining large object types used by NuoDB_Lob */
enum NuoDB_Lob_Type_enum {

/**
 * @brief Representation of a Binary Large Object value to be stored
 * as a column value in a row of a table.
 *
 * A SQL BLOB is a built-in data type that stores a Binary Large
 * Object as a column value in a row of a database table.
 *
 * See <a href="http://doc.nuodb.com/Latest/Default.htm#SQL-Binary-Types.htm"><strong>Binary Data Types</strong></a>
 * in the NuoDB Documentation for more information about this data type.
 */
    NUODB_BLOB_TYPE = 1,


/**
 * @brief Representation of a Character Large Object (CLOB) value to be
 * stored as a column value in a row of a table.
 *
 * A SQL CLOB is a built-in data type that stores a Character Large
 * Object as a column value in a row of a database table.
 */
    NUODB_CLOB_TYPE = 2,

/**
 * @brief Representation of Binary to be stored as a column value in a
 * row of a table.
 *
 * A SQL BINARY is a built-in data type that stores a Bytes as a
 * column value in a row of a database table.
 */
    NUODB_BYTES_TYPE = 3,

};

/*! Large object types used by NuoDB_Lob. */
typedef enum NuoDB_Lob_Type_enum  NuoDB_Lob_Type;


/*! %NuoDB Large Object
 *
 * NuoDB_Lob is a container which is used to represent any of the
 * following SQL Lob types:
 *
 * @li Blob
 * @li Clob
 * @li Bytes
 *
 */
struct NuoDB_Lob
{
    /** @internal */
    _NuoDB_Lob_state_t* _state;

    /**
     * @brief   Returns the current length of the Lob data.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  length of Lob data in bytes.
     */
    size_t (* getLength) (NuoDB_Lob* _this);

    /**
     * @brief   Gets a pointer to Lob data.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  a pointer to the Lob data as an array of bytes.
     */
    const char*   (* getData) (NuoDB_Lob* _this);

    /**
     * @brief   Sets a pointer to client's data buffer.
     *
     * Makes a shallow copy of the client data pointer.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   buffer  Client data pointer as an array of bytes.
     * @param[in]   len     Data length in bytes.
     *
     * @return  Enumerated integer NuoDB_Status.  NUODB_SUCCESS
     * (integer 0) on success, or a negative integer on failure.
     */
    NuoDB_Status (* setData) (NuoDB_Lob* _this, const char* buffer, size_t len);



    /**
     * @brief   Returns the type of Lob data.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  pointer to Lob type.
     */
    NuoDB_Lob_Type   (* getType) (NuoDB_Lob* _this);

};

/**
 * @brief   Create a NuoDB_Lob data structure.
 *
 * NuoDB_Lob is a container for various Large Objects.  Clients must
 * create a NuoDB_Lob data structure when passing any of the
 * following Large Object types to a prepared statement.  It is the
 * responsibility of the caller to call NuoDB_Lob_free() when
 * the caller is finished with it.
 *
 * @param[in]   connection    a pointer to the connection.
 * @param[in]   type          valid types are:
 * <ul>
 * <li>NUODB_BLOB_TYPE  = 1</li>
 * <li>NUODB_CLOB_TYPE  = 2</li>
 * <li>NUODB_BYTES_TYPE = 3</li>
 * </ul>
 *
 * @return  a pointer to a NuoDB_Lob data structure
 */
NUODB_CAPI NuoDB_Lob*  NUODB_CAPICALL NuoDB_Lob_create(NuoDB_Connection* connection, NuoDB_Lob_Type type);

/**
 * @brief   Free a NuoDB_Lob data structure.
 *
 * Clients should call this function to free the NuoDB_Lob.
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Lob_free(NuoDB_Lob* lob);

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* end of: #ifndef _NUODB_LOB_H_ */
