/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NuoDB_TEMPORAL_H_
#define _NuoDB_TEMPORAL_H_ 1

#include <time.h>

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file temporal.h
 * @brief Data structure used to represent temporal information.
 */

/*! Enum defining constant temporal types used by NuoDB_Temporal. */
enum NuoDB_Temporal_Type_enum {
        /**
         * @brief   SQL DATE type
         */
        NUODB_TEMPORAL_DATE         = 1,

        /**
         * @brief   SQL TIME type
         */
        NUODB_TEMPORAL_TIME         = 2,

        /**
         * @brief   SQL TIMESTAMP type
         */
        NUODB_TEMPORAL_TIMESTAMP    = 3,

};

/*! Constant temporal types used by NuoDB_Temporal. */
typedef enum NuoDB_Temporal_Type_enum  NuoDB_Temporal_Type;

/*! Data structure used to represent temporal information.
 *
 * The NuoDB_Temporal data structure can be used to store and retrieve any
 * NuoDB Temporal information (SQL DATE, SQL TIME, or SQL TIMESTAMP).
 */
struct NuoDB_Temporal
{
    NuoDB_Temporal_Type type;
    int32_t nanoseconds;
    int64_t milliseconds;

    /**
     * @brief   Return the SQL type for the NuoDB_Temporal data structure.
     *
     * The NuoDB_Temporal data structure can be used to store and retrieve a
     * SQL DATE, SQL TIME, or SQL TIMESTAMP.  This function returns
     * the SQL type.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  the SQL type.
     */
    int     (* getType)(NuoDB_Temporal* _this);

    /**
     * @brief   Set the NuoDB_Temporal data structure with the
     * specified number of milliseconds.
     *
     * @param[in]   _this     a pointer to the current (this) data structure.
     * @param[in]   millis    a 64-bit integer specifying the number of
     *                    milliseconds since the common (UNIX) epoch,
     *                    January 1, 1970
     */
    void    (* setMilliSeconds)(NuoDB_Temporal* _this, int64_t millis);

    /**
     * @brief   Return the value of the corresponding NuoDB_Temporal
     * data structure as milliseconds.
     *
     * The milliseconds returned will be UTC milliseconds since the common
     * (UNIX) epoch, Jan 1, 1970.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     *
     * @return  the number of milliseconds in the SQL TIMESTAMP
     */
    int64_t (* getMilliSeconds)(NuoDB_Temporal* _this);

    /**
     * @brief   Set the fractions of a second in the NuoDB_Temporal
     * data structure with the specified number of nanoseconds.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   nanos   a 32-bit integer specifying the fractional
     * part of one second in nanoseconds.
     */
    void    (* setNanoSeconds)(NuoDB_Temporal* _this, int32_t nanos);

    /**
     * @brief   Return the value of the fractional part of the
     *          corresponding NuoDB_Temporal data structure as nanoseconds
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @return  the number of nanoseconds in the fraction of seconds
     *          in the NuoDB_Temporal data structure.
     */
    int32_t (* getNanoSeconds)(NuoDB_Temporal* _this);

    /**
     * @brief   Set this timestamp from the time specified in utc_time
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   utc_time    C standard tm struct (<time.h>)
     */
    void    (* setTimestamp)(NuoDB_Temporal* _this, struct tm* utc_time);

    /**
     * @brief   Populate the utc_time struct with the value from this
     *          timestamp based on the UTC time zone (versus the local
     *          time zone).
     *
     * @param[in]   _this     a pointer to the current (this) data structure.
     * @param[out]  utc_time  C standard tm time struct (<time.h>)
     */
    void    (* getTimestamp)(NuoDB_Temporal* _this, struct tm* utc_time);

    /**
     * @brief   Set this timestamp from the time specified in tz_time
     *          using the time zone being passed in.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   timeZoneId  %NuoDB uses ICU for timezone
     * conversions. timezoneId is an id for a time zone, such as
     * "America/Los_Angeles", or a custom ID such as "GMT-8:00".  It
     * is passed as an argument to
     *  <a href="http://icu-project.org/apiref/icu4c/classicu_1_1TimeZone.html#a35da0507b62754ffe5d8d59c19775cdb"><strong>icu::TimeZone::createTimeZone()</strong></a>.
     * @param[in]   tz_time     C standard tm struct (<time.h>)
     */
    void    (* setTimestampTZ)(NuoDB_Temporal* _this, const char* timeZoneId, struct tm* tz_time);

    /**
     * @brief   Populate the tm time struct with the value from this
     *          timestamp based on the time zone being passed in.
     *
     * @param[in]   _this   a pointer to the current (this) data structure.
     * @param[in]   timeZoneId  %NuoDB uses ICU for timezone
     * conversions. timezoneId is an id for a time zone, such as
     * "America/Los_Angeles", or a custom ID such as "GMT-8:00".  It
     * is passed as an argument to
     *                      <a href="http://icu-project.org/apiref/icu4c/classicu_1_1TimeZone.html#a35da0507b62754ffe5d8d59c19775cdb"><strong>icu::TimeZone::createTimeZone()</strong></a>.
     * @param[out]   tm_time     C standard tm time struct (<time.h>)
     */
    void    (* getTimestampTZ)(NuoDB_Temporal* _this, const char* timeZoneId, struct tm* tz_time);

    /**
     * @brief   Set this timestamp from the time specified in tz_time
     *          using the time zone being passed in.
     *
     * @param[in]   _this     a pointer to the current (this) data structure.
     * @param[in]   timeZone  pointer to an initialized timezone
     * @param[in]   tz_time   C standard tm struct (<time.h>)
     */
    void    (* setTimestampNuoDBTimezone)(NuoDB_Temporal* _this, const NuoDB_Timezone* timeZone, struct tm* tz_time);

    /**
     * @brief   Populate the tm time struct with the value from this
     *          timestamp based on the time zone being passed in.
     *
     * @param[in]   _this    a pointer to the current (this) data structure.
     * @param[in]   timeZone pointer to an initialized timezone
     * @param[out]  tm_time  C standard tm time struct (<time.h>)
     */
    void    (* getTimestampNuoDBTimezone)(NuoDB_Temporal* _this, const NuoDB_Timezone* timeZone, struct tm* tz_time);
};


/**
 * @brief   Create a NuoDB_Timezone data structure for the specified
 * timezone id.
 *
 * The client is expected to call NuoDB_Timezone_free() when it is no
 * longer using the data structure.
 */
NUODB_CAPI const NuoDB_Timezone*  NUODB_CAPICALL NuoDB_Timezone_create(const char* timezoneId);

/**
 * @brief   Free the NuoDB_Timezone data structure.
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Timezone_free(const NuoDB_Timezone* timezone);

/**
 * @brief   Create a NuoDB_Temporal data structure for the specified
 * SQL type.
 *
 * The client is expected to call NuoDB_Temporal_free() when it is no
 * longer using the NuoDB_Temporal data structure.
 */
NUODB_CAPI NuoDB_Temporal*  NUODB_CAPICALL  NuoDB_Temporal_create(NuoDB_Temporal_Type temporalType);

/**
 * @brief   Initialize a NuoDB_Temporal data structure with the specified
 * SQL type.
 */
NUODB_CAPI void NUODB_CAPICALL NuoDB_Temporal_init(NuoDB_Temporal* tm, NuoDB_Temporal_Type temporalType);

/**
 * @brief   Free the NuoDB_Temporal data structure.
 */
NUODB_CAPI void NUODB_CAPICALL              NuoDB_Temporal_free(NuoDB_Temporal* temporal);

#ifdef __cplusplus
} // extern "C" {
#endif

#endif /* end of: #ifndef _NuoDB_TEMPORAL_H_ */
