/*
 * Copyright 2015 NuoDB, Inc. All Rights Reserved.
 *
 * An introductory program that shows how to establish a connection with
 * a NuoDB server and execute a few basic SQL commands.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "NuoDB.h"

using namespace NuoDB;
#include "HelloDB.h"

/*
 * Creates an instance of HelloDB connected to the database with
 * the given name on the localhost. This example class uses the
 * default testing name & password.
 */
HelloDB::HelloDB(const char* dbName) throw (SQLException)
{
    connection =
        Connection::create(dbName,
                           "dba", // username
                           "goalie",  // password
                           1,   // number of properties
                           "schema", "hello");  // schema=hello property
}

/*
 * Closes the connection.
 */
HelloDB::~HelloDB()
{
    connection->close();
}

/*
 * Creates a testing table named "names" which maps integer identifiers
 * to names.
 */
void HelloDB::createTable() throw (SQLException)
{
    Statement* stmt = connection->createStatement();

    try {
        stmt->execute("create table names (id bigint generated always as identity primary key, name string)");
        connection->commit();
        stmt->close();
        printf("The table 'NAMES' was created.\n");
    } catch (SQLException& xcp) {
        connection->rollback();
        printf("The table 'NAMES' already exists, re-using it.\n");
    }
}

/*
 * Adds an identifier-to-name mapping in the "names" table.
 */
void HelloDB::addNames() throw (SQLException)
{
    PreparedStatement* stmt =
        connection->prepareStatement("insert into names (name) values (?),(?),(?),(?),(?),(?),(?),(?),(?),(?),(?),(?),(?),(?),(?)", NuoDB::RETURN_GENERATED_KEYS);

    /* batch insert a set of names */
    for (int i = 1; i <= 15; i++) {
        char name[100];
        sprintf(name, "Fred # %d", i);
        stmt->setString(i, name);
    }

    try {
        int updateCount = stmt->executeUpdate();

        // This is how to retrieve the generated key. Only the generated column value is returned

        ResultSet*          resultSet = stmt->getGeneratedKeys();
        ResultSetMetaData*  resultSetMetaData = resultSet->getMetaData();
        const char*         columnName = resultSetMetaData->getColumnName(1);

        while (resultSet->next()) {
            printf("New id=%d for column %s\n", resultSet->getInt(1), columnName);
        }

        connection->commit();
        resultSet->close();
        stmt->close();
    } catch (SQLException& xcp) {
        connection->rollback();
        throw;
    }
}

/*
 * Gets the name mapped to the given identifier. Returns NULL if there
 * is no such identifier in the table.
 */
const char* HelloDB::getName(int id) throw (SQLException)
{
    PreparedStatement* stmt =
        connection->prepareStatement("select name from names where id=?");

    stmt->setInt(1, id);

    ResultSet*  rs = stmt->executeQuery();
    char*       name;

    if (rs->next()) {
        name = new char[strlen(rs->getString(1)) + 1];
        strcpy(name, rs->getString(1));
    } else {
        name = NULL;
    }

    rs->close();
    stmt->close();

    return name;
}

int main(int argc, char** argv)
{
    if (argc != 2) {
        printf("Usage: %s <dbname>\n", argv[0]);
        return -1;
    }

    try {
        HelloDB helloDB(argv[1]);

        helloDB.createTable();
        helloDB.addNames();
        const char* name = helloDB.getName(12);

        printf("The NAME with id of 12 is '%s'\n", name);

        delete[] name;
    } catch (SQLException& xcp) {
        printf("Got exception: %s\n", xcp.getText());
        return -1;
    }

    return 0;
}
